from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.event_status import EventStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.mcp_call_event_arguments_type_0 import MCPCallEventArgumentsType0
    from ..models.mcp_call_event_metadata_type_0 import MCPCallEventMetadataType0
    from ..models.mcp_call_event_result_type_0 import MCPCallEventResultType0


T = TypeVar("T", bound="MCPCallEvent")


@_attrs_define
class MCPCallEvent:
    """A Model Context Protocol (MCP) tool call.

    MCP is a protocol for connecting LLMs to external tools/data sources.
    This is distinct from internal tools because it involves external integrations.

    Attributes
    ----------
            type_ (Union[Literal['mcp_call'], Unset]):  Default: 'mcp_call'.
            id (Union[None, Unset, str]): Unique identifier for the event
            status (Union[EventStatus, None, Unset]): Status of the event
            metadata (Union['MCPCallEventMetadataType0', None, Unset]): Provider-specific metadata and additional fields
            error_message (Union[None, Unset, str]): Error message if the event failed
            tool_name (Union[None, Unset, str]): Name of the MCP tool being called
            server_name (Union[None, Unset, str]): Name of the MCP server
            arguments (Union['MCPCallEventArgumentsType0', None, Unset]): Arguments for the MCP tool call
            result (Union['MCPCallEventResultType0', None, Unset]): Result from the MCP tool call
    """

    type_: Union[Literal["mcp_call"], Unset] = "mcp_call"
    id: Union[None, Unset, str] = UNSET
    status: Union[EventStatus, None, Unset] = UNSET
    metadata: Union["MCPCallEventMetadataType0", None, Unset] = UNSET
    error_message: Union[None, Unset, str] = UNSET
    tool_name: Union[None, Unset, str] = UNSET
    server_name: Union[None, Unset, str] = UNSET
    arguments: Union["MCPCallEventArgumentsType0", None, Unset] = UNSET
    result: Union["MCPCallEventResultType0", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.mcp_call_event_arguments_type_0 import MCPCallEventArgumentsType0
        from ..models.mcp_call_event_metadata_type_0 import MCPCallEventMetadataType0
        from ..models.mcp_call_event_result_type_0 import MCPCallEventResultType0

        type_ = self.type_

        id: Union[None, Unset, str]
        id = UNSET if isinstance(self.id, Unset) else self.id

        status: Union[None, Unset, str]
        if isinstance(self.status, Unset):
            status = UNSET
        elif isinstance(self.status, EventStatus):
            status = self.status.value
        else:
            status = self.status

        metadata: Union[None, Unset, dict[str, Any]]
        if isinstance(self.metadata, Unset):
            metadata = UNSET
        elif isinstance(self.metadata, MCPCallEventMetadataType0):
            metadata = self.metadata.to_dict()
        else:
            metadata = self.metadata

        error_message: Union[None, Unset, str]
        error_message = UNSET if isinstance(self.error_message, Unset) else self.error_message

        tool_name: Union[None, Unset, str]
        tool_name = UNSET if isinstance(self.tool_name, Unset) else self.tool_name

        server_name: Union[None, Unset, str]
        server_name = UNSET if isinstance(self.server_name, Unset) else self.server_name

        arguments: Union[None, Unset, dict[str, Any]]
        if isinstance(self.arguments, Unset):
            arguments = UNSET
        elif isinstance(self.arguments, MCPCallEventArgumentsType0):
            arguments = self.arguments.to_dict()
        else:
            arguments = self.arguments

        result: Union[None, Unset, dict[str, Any]]
        if isinstance(self.result, Unset):
            result = UNSET
        elif isinstance(self.result, MCPCallEventResultType0):
            result = self.result.to_dict()
        else:
            result = self.result

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type_ is not UNSET:
            field_dict["type"] = type_
        if id is not UNSET:
            field_dict["id"] = id
        if status is not UNSET:
            field_dict["status"] = status
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if error_message is not UNSET:
            field_dict["error_message"] = error_message
        if tool_name is not UNSET:
            field_dict["tool_name"] = tool_name
        if server_name is not UNSET:
            field_dict["server_name"] = server_name
        if arguments is not UNSET:
            field_dict["arguments"] = arguments
        if result is not UNSET:
            field_dict["result"] = result

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.mcp_call_event_arguments_type_0 import MCPCallEventArgumentsType0
        from ..models.mcp_call_event_metadata_type_0 import MCPCallEventMetadataType0
        from ..models.mcp_call_event_result_type_0 import MCPCallEventResultType0

        d = dict(src_dict)
        type_ = cast(Union[Literal["mcp_call"], Unset], d.pop("type", UNSET))
        if type_ != "mcp_call" and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'mcp_call', got '{type_}'")

        def _parse_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        id = _parse_id(d.pop("id", UNSET))

        def _parse_status(data: object) -> Union[EventStatus, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                return EventStatus(data)

            except:  # noqa: E722
                pass
            return cast(Union[EventStatus, None, Unset], data)

        status = _parse_status(d.pop("status", UNSET))

        def _parse_metadata(data: object) -> Union["MCPCallEventMetadataType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return MCPCallEventMetadataType0.from_dict(data)

            except:  # noqa: E722
                pass
            return cast(Union["MCPCallEventMetadataType0", None, Unset], data)

        metadata = _parse_metadata(d.pop("metadata", UNSET))

        def _parse_error_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        error_message = _parse_error_message(d.pop("error_message", UNSET))

        def _parse_tool_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        tool_name = _parse_tool_name(d.pop("tool_name", UNSET))

        def _parse_server_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        server_name = _parse_server_name(d.pop("server_name", UNSET))

        def _parse_arguments(data: object) -> Union["MCPCallEventArgumentsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return MCPCallEventArgumentsType0.from_dict(data)

            except:  # noqa: E722
                pass
            return cast(Union["MCPCallEventArgumentsType0", None, Unset], data)

        arguments = _parse_arguments(d.pop("arguments", UNSET))

        def _parse_result(data: object) -> Union["MCPCallEventResultType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return MCPCallEventResultType0.from_dict(data)

            except:  # noqa: E722
                pass
            return cast(Union["MCPCallEventResultType0", None, Unset], data)

        result = _parse_result(d.pop("result", UNSET))

        mcp_call_event = cls(
            type_=type_,
            id=id,
            status=status,
            metadata=metadata,
            error_message=error_message,
            tool_name=tool_name,
            server_name=server_name,
            arguments=arguments,
            result=result,
        )

        mcp_call_event.additional_properties = d
        return mcp_call_event

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.llm_export_format import LLMExportFormat
from ..models.root_type import RootType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.log_records_boolean_filter import LogRecordsBooleanFilter
    from ..models.log_records_collection_filter import LogRecordsCollectionFilter
    from ..models.log_records_date_filter import LogRecordsDateFilter
    from ..models.log_records_id_filter import LogRecordsIDFilter
    from ..models.log_records_number_filter import LogRecordsNumberFilter
    from ..models.log_records_sort_clause import LogRecordsSortClause
    from ..models.log_records_text_filter import LogRecordsTextFilter


T = TypeVar("T", bound="LogRecordsExportRequest")


@_attrs_define
class LogRecordsExportRequest:
    """Request schema for exporting log records (sessions, traces, spans).

    Attributes
    ----------
        root_type (RootType):
        log_stream_id (Union[None, Unset, str]): Log stream id associated with the traces.
        experiment_id (Union[None, Unset, str]): Experiment id associated with the traces.
        metrics_testing_id (Union[None, Unset, str]): Metrics testing id associated with the traces.
        filters (Union[Unset, list[Union['LogRecordsBooleanFilter', 'LogRecordsCollectionFilter',
            'LogRecordsDateFilter', 'LogRecordsIDFilter', 'LogRecordsNumberFilter', 'LogRecordsTextFilter']]]): Filters to
            apply on the export
        sort (Union['LogRecordsSortClause', None, Unset]): Sort clause for the export.  Defaults to native sort
            (created_at, id descending).
        column_ids (Union[None, Unset, list[str]]): Column IDs to include in export
        export_format (Union[Unset, LLMExportFormat]):
        redact (Union[Unset, bool]): Redact sensitive data Default: True.
        file_name (Union[None, Unset, str]): Optional filename for the exported file
    """

    root_type: RootType
    log_stream_id: Union[None, Unset, str] = UNSET
    experiment_id: Union[None, Unset, str] = UNSET
    metrics_testing_id: Union[None, Unset, str] = UNSET
    filters: Union[
        Unset,
        list[
            Union[
                "LogRecordsBooleanFilter",
                "LogRecordsCollectionFilter",
                "LogRecordsDateFilter",
                "LogRecordsIDFilter",
                "LogRecordsNumberFilter",
                "LogRecordsTextFilter",
            ]
        ],
    ] = UNSET
    sort: Union["LogRecordsSortClause", None, Unset] = UNSET
    column_ids: Union[None, Unset, list[str]] = UNSET
    export_format: Union[Unset, LLMExportFormat] = UNSET
    redact: Union[Unset, bool] = True
    file_name: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.log_records_boolean_filter import LogRecordsBooleanFilter
        from ..models.log_records_collection_filter import LogRecordsCollectionFilter
        from ..models.log_records_date_filter import LogRecordsDateFilter
        from ..models.log_records_id_filter import LogRecordsIDFilter
        from ..models.log_records_number_filter import LogRecordsNumberFilter
        from ..models.log_records_sort_clause import LogRecordsSortClause

        root_type = self.root_type.value

        log_stream_id: Union[None, Unset, str]
        log_stream_id = UNSET if isinstance(self.log_stream_id, Unset) else self.log_stream_id

        experiment_id: Union[None, Unset, str]
        experiment_id = UNSET if isinstance(self.experiment_id, Unset) else self.experiment_id

        metrics_testing_id: Union[None, Unset, str]
        metrics_testing_id = UNSET if isinstance(self.metrics_testing_id, Unset) else self.metrics_testing_id

        filters: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item: dict[str, Any]
                if isinstance(
                    filters_item_data,
                    (
                        LogRecordsIDFilter,
                        LogRecordsDateFilter,
                        LogRecordsNumberFilter,
                        LogRecordsBooleanFilter,
                        LogRecordsCollectionFilter,
                    ),
                ):
                    filters_item = filters_item_data.to_dict()
                else:
                    filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        sort: Union[None, Unset, dict[str, Any]]
        if isinstance(self.sort, Unset):
            sort = UNSET
        elif isinstance(self.sort, LogRecordsSortClause):
            sort = self.sort.to_dict()
        else:
            sort = self.sort

        column_ids: Union[None, Unset, list[str]]
        if isinstance(self.column_ids, Unset):
            column_ids = UNSET
        elif isinstance(self.column_ids, list):
            column_ids = self.column_ids

        else:
            column_ids = self.column_ids

        export_format: Union[Unset, str] = UNSET
        if not isinstance(self.export_format, Unset):
            export_format = self.export_format.value

        redact = self.redact

        file_name: Union[None, Unset, str]
        file_name = UNSET if isinstance(self.file_name, Unset) else self.file_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"root_type": root_type})
        if log_stream_id is not UNSET:
            field_dict["log_stream_id"] = log_stream_id
        if experiment_id is not UNSET:
            field_dict["experiment_id"] = experiment_id
        if metrics_testing_id is not UNSET:
            field_dict["metrics_testing_id"] = metrics_testing_id
        if filters is not UNSET:
            field_dict["filters"] = filters
        if sort is not UNSET:
            field_dict["sort"] = sort
        if column_ids is not UNSET:
            field_dict["column_ids"] = column_ids
        if export_format is not UNSET:
            field_dict["export_format"] = export_format
        if redact is not UNSET:
            field_dict["redact"] = redact
        if file_name is not UNSET:
            field_dict["file_name"] = file_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.log_records_boolean_filter import LogRecordsBooleanFilter
        from ..models.log_records_collection_filter import LogRecordsCollectionFilter
        from ..models.log_records_date_filter import LogRecordsDateFilter
        from ..models.log_records_id_filter import LogRecordsIDFilter
        from ..models.log_records_number_filter import LogRecordsNumberFilter
        from ..models.log_records_sort_clause import LogRecordsSortClause
        from ..models.log_records_text_filter import LogRecordsTextFilter

        d = dict(src_dict)
        root_type = RootType(d.pop("root_type"))

        def _parse_log_stream_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        log_stream_id = _parse_log_stream_id(d.pop("log_stream_id", UNSET))

        def _parse_experiment_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        experiment_id = _parse_experiment_id(d.pop("experiment_id", UNSET))

        def _parse_metrics_testing_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        metrics_testing_id = _parse_metrics_testing_id(d.pop("metrics_testing_id", UNSET))

        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:

            def _parse_filters_item(
                data: object,
            ) -> Union[
                "LogRecordsBooleanFilter",
                "LogRecordsCollectionFilter",
                "LogRecordsDateFilter",
                "LogRecordsIDFilter",
                "LogRecordsNumberFilter",
                "LogRecordsTextFilter",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return LogRecordsIDFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return LogRecordsDateFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return LogRecordsNumberFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return LogRecordsBooleanFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return LogRecordsCollectionFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                return LogRecordsTextFilter.from_dict(data)

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        def _parse_sort(data: object) -> Union["LogRecordsSortClause", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return LogRecordsSortClause.from_dict(data)

            except:  # noqa: E722
                pass
            return cast(Union["LogRecordsSortClause", None, Unset], data)

        sort = _parse_sort(d.pop("sort", UNSET))

        def _parse_column_ids(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                return cast(list[str], data)

            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        column_ids = _parse_column_ids(d.pop("column_ids", UNSET))

        _export_format = d.pop("export_format", UNSET)
        export_format: Union[Unset, LLMExportFormat]
        export_format = UNSET if isinstance(_export_format, Unset) else LLMExportFormat(_export_format)

        redact = d.pop("redact", UNSET)

        def _parse_file_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        file_name = _parse_file_name(d.pop("file_name", UNSET))

        log_records_export_request = cls(
            root_type=root_type,
            log_stream_id=log_stream_id,
            experiment_id=experiment_id,
            metrics_testing_id=metrics_testing_id,
            filters=filters,
            sort=sort,
            column_ids=column_ids,
            export_format=export_format,
            redact=redact,
            file_name=file_name,
        )

        log_records_export_request.additional_properties = d
        return log_records_export_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Generator
from datetime import datetime
from typing import Any, Optional, Union

from galileo import GalileoLogger
from galileo.openai.extractors import (
    convert_to_galileo_message,
    extract_streamed_openai_response,
    has_pending_function_calls,
    process_function_call_outputs,
    process_output_items,
)
from galileo.openai.models import OpenAiInputData, OpenAiModuleDefinition
from galileo.utils import _get_timestamp
from galileo.utils.serialization import serialize_to_str

try:
    import openai
except ImportError:
    AsyncAzureOpenAI = None  # type: ignore[assignment]
    AsyncOpenAI = None  # type: ignore[assignment]
    AzureOpenAI = None  # type: ignore[assignment]
    OpenAI = None  # type: ignore[assignment]


class ResponseGeneratorSync:
    """
    A wrapper for OpenAI streaming responses that logs the response to Galileo.

    This class wraps the OpenAI streaming response generator and logs the response
    to Galileo when the generator is exhausted. It implements the iterator protocol
    to allow for streaming responses.

    Attributes
    ----------
    resource : OpenAiModuleDefinition
        The OpenAI resource definition.
    response : Generator or openai.Stream
        The OpenAI streaming response.
    input_data : OpenAiInputData
        The input data for the OpenAI request.
    logger : GalileoLogger
        The Galileo logger instance.
    should_complete_trace : bool
        Whether to complete the trace when the generator is exhausted.
    """

    def __init__(
        self,
        *,
        resource: OpenAiModuleDefinition,
        response: Union[Generator, openai.Stream],
        input_data: OpenAiInputData,
        logger: GalileoLogger,
        should_complete_trace: bool,
        status_code: int = 200,
    ):
        self.items: list[Any] = []
        self.resource = resource
        self.response = response
        self.input_data = input_data
        self.logger = logger
        self.should_complete_trace = should_complete_trace
        self.completion_start_time: Optional[datetime] = None
        self.status_code = status_code

    def __iter__(self):
        try:
            for i in self.response:
                self.items.append(i)

                if self.completion_start_time is None:
                    self.completion_start_time = _get_timestamp()

                yield i
        finally:
            self._finalize()

    def __next__(self):
        try:
            item = self.response.__next__()
            self.items.append(item)

            if self.completion_start_time is None:
                self.completion_start_time = _get_timestamp()

            return item

        except StopIteration:
            self._finalize()

            raise

    def __enter__(self):
        return self.__iter__()

    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None:
        pass

    def _finalize(self) -> None:
        model, completion, usage = extract_streamed_openai_response(self.resource, self.items)

        if usage is None:
            usage = {}

        end_time = _get_timestamp()
        # TODO: make sure completion_start_time what we want
        duration_ns = (
            round((end_time - self.completion_start_time).total_seconds() * 1e9) if self.completion_start_time else 0
        )

        if isinstance(self.input_data.input, list):
            span_input = [convert_to_galileo_message(msg) for msg in self.input_data.input]
        else:
            span_input = [convert_to_galileo_message(self.input_data.input)]

        # probably can create a shared function for handling both streaming and non-streaming
        # Process Responses API output items sequentially if present (same as non-streaming)
        final_conversation_context = span_input.copy()
        output_items: list = []
        if self.resource.type == "response" and completion:
            # First, process any function_call_output items in the input to create tool spans
            # This represents tool executions that happened before this API call
            if isinstance(self.input_data.input, list):
                process_function_call_outputs(self.input_data.input, self.logger)

            # For streaming Responses API, we need to extract output items from the completion
            # The completion should contain the final response with output items
            if isinstance(completion, dict) and "output" in completion:
                output_items = completion.get("output", [])

                # Process all output items sequentially and get the final context
                final_conversation_context = process_output_items(
                    output_items,
                    self.logger,
                    model,
                    span_input,
                    self.input_data.model_parameters,
                    status_code=self.status_code,
                    tools=self.input_data.tools,
                    usage=usage,
                )
            else:
                # Fallback: create basic span if no output items
                span_output = convert_to_galileo_message(completion, "assistant")
                span = self.logger.add_llm_span(
                    input=span_input,
                    output=span_output,
                    tools=self.input_data.tools,
                    name=self.input_data.name,
                    model=model,
                    temperature=self.input_data.temperature,
                    duration_ns=duration_ns,
                    num_input_tokens=usage.get("input_tokens", 0),
                    num_output_tokens=usage.get("output_tokens", 0),
                    total_tokens=usage.get("total_tokens", 0),
                    metadata={str(k): str(v) for k, v in self.input_data.model_parameters.items()},
                    status_code=self.status_code,
                )
                span.metrics.num_reasoning_tokens = usage.get("reasoning_tokens", 0) if usage else 0
                span.metrics.num_cached_input_tokens = usage.get("cached_tokens", 0) if usage else 0
        else:
            # For non-Responses API (chat or completion), create the main span as before
            span_output = convert_to_galileo_message(completion, "assistant")

            # Add a span to the current trace or span (if this is a nested trace)
            span = self.logger.add_llm_span(
                input=span_input,
                output=span_output,
                tools=self.input_data.tools,
                name=self.input_data.name,
                model=model,
                temperature=self.input_data.temperature,
                duration_ns=duration_ns,
                num_input_tokens=usage.get("input_tokens", 0),
                num_output_tokens=usage.get("output_tokens", 0),
                total_tokens=usage.get("total_tokens", 0),
                metadata={str(k): str(v) for k, v in self.input_data.model_parameters.items()},
                status_code=self.status_code,
            )
            span.metrics.num_reasoning_tokens = usage.get("reasoning_tokens", 0) if usage else 0
            span.metrics.num_cached_input_tokens = usage.get("cached_tokens", 0) if usage else 0

        # Conclude the trace if this is the top-level call
        # For Responses API: don't conclude if there are pending function calls (model waiting for tool results)
        has_pending_calls = (
            self.resource.type == "response" and output_items and has_pending_function_calls(output_items)
        )

        if self.should_complete_trace and not has_pending_calls:
            if self.resource.type == "response":
                # For Responses API, use the final conversation context from processing
                full_conversation = final_conversation_context
            else:
                # For other APIs, add the final span output
                full_conversation = []
                if isinstance(self.input_data.input, list):
                    full_conversation.extend([convert_to_galileo_message(msg) for msg in self.input_data.input])
                else:
                    full_conversation.append(convert_to_galileo_message(self.input_data.input))
                full_conversation.append(convert_to_galileo_message(completion, "assistant"))

            # Serialize with "messages" wrapper for UI compatibility
            trace_output = {"messages": [msg.model_dump(exclude_none=True) for msg in full_conversation]}
            self.logger.conclude(
                output=serialize_to_str(trace_output), duration_ns=duration_ns, status_code=self.status_code
            )



import contextlib
import os
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Callable, Optional

from galileo.__future__.shared.exceptions import ConfigurationError
from galileo.config import GalileoPythonConfig
from galileo.utils.logging import get_logger

logger = get_logger(__name__)


@dataclass(frozen=True)
class ConfigKey:
    """
    Metadata for a configuration key.

    Defines a single configuration option with its properties, validation rules,
    and relationship to environment variables. Used by the ConfigurationMeta
    metaclass to enable dynamic attribute access.

    Attributes
    ----------
    name : str
        The attribute name used in the Configuration class (e.g., "galileo_api_key").
    env_var : str
        The corresponding environment variable name (e.g., "GALILEO_API_KEY").
    description : str
        Human-readable description of the configuration key's purpose.
    required : bool
        Whether this key must be set for the configuration to be considered complete.
        Default: False.
    sensitive : bool
        Whether this key contains sensitive information (e.g., API keys, passwords).
        Sensitive values are masked in get_configuration() output. Default: False.
    default : Any
        The default value if no explicit value or environment variable is set.
        Default: None.
    value_type : type
        The expected Python type for this configuration value. Default: str.
    parser : Optional[Callable[[str], Any]]
        Optional function to convert string values from environment variables
        to the appropriate type. Default: None (no conversion).
    """

    name: str
    env_var: str
    description: str
    required: bool = False
    sensitive: bool = False
    default: Any = None
    value_type: type = str
    parser: Optional[Callable[[str], Any]] = None


_CONFIGURATION_KEYS = [
    ConfigKey(
        name="galileo_api_key",
        env_var="GALILEO_API_KEY",
        description="API key for authenticating with Galileo",
        required=True,
        sensitive=True,
    ),
    ConfigKey(
        name="console_url", env_var="GALILEO_CONSOLE_URL", description="URL of the Galileo console", required=True
    ),
    ConfigKey(
        name="openai_api_key",
        env_var="OPENAI_API_KEY",
        description="OpenAI API key for interoperability with OpenAI SDK",
        sensitive=True,
    ),
    ConfigKey(name="default_project_name", env_var="GALILEO_PROJECT", description="Default project name"),
    ConfigKey(name="default_project_id", env_var="GALILEO_PROJECT_ID", description="Default project ID"),
    ConfigKey(name="default_logstream_name", env_var="GALILEO_LOG_STREAM", description="Default log stream name"),
    ConfigKey(name="default_logstream_id", env_var="GALILEO_LOG_STREAM_ID", description="Default log stream ID"),
    ConfigKey(
        name="default_scorer_model",
        env_var="GALILEO_DEFAULT_SCORER_MODEL",
        description="Default model for LLM-based scorers/metrics",
        default="gpt-4.1-mini",
    ),
    ConfigKey(
        name="default_scorer_judges",
        env_var="GALILEO_DEFAULT_SCORER_JUDGES",
        description="Default number of judges for LLM-based scorers/metrics",
        default=3,
        value_type=int,
        parser=int,
    ),
    ConfigKey(
        name="logging_disabled",
        env_var="GALILEO_LOGGING_DISABLED",
        description="Disable all logging to Galileo",
        default=False,
        value_type=bool,
        parser=lambda v: v.lower() in ("true", "1", "t", "yes"),
    ),
    ConfigKey(
        name="code_validation_timeout",
        env_var="GALILEO_CODE_VALIDATION_TIMEOUT",
        description="Timeout in seconds for code scorer validation",
        default=60.0,
        value_type=float,
        parser=float,
    ),
    ConfigKey(
        name="code_validation_initial_delay",
        env_var="GALILEO_CODE_VALIDATION_INITIAL_DELAY",
        description="Initial delay in seconds between validation polling attempts",
        default=5.0,
        value_type=float,
        parser=float,
    ),
    ConfigKey(
        name="code_validation_max_delay",
        env_var="GALILEO_CODE_VALIDATION_MAX_DELAY",
        description="Maximum delay in seconds between validation polling attempts",
        default=30.0,
        value_type=float,
        parser=float,
    ),
    ConfigKey(
        name="code_validation_backoff_multiplier",
        env_var="GALILEO_CODE_VALIDATION_BACKOFF_MULTIPLIER",
        description="Multiplier for exponential backoff between validation polling attempts",
        default=1.5,
        value_type=float,
        parser=float,
    ),
]

_KEYS_BY_NAME = {key.name: key for key in _CONFIGURATION_KEYS}


class ConfigurationMeta(type):
    """
    Metaclass for dynamic attribute handling based on CONFIGURATION_KEYS.

    This metaclass enables the Configuration class to provide dynamic attribute access
    with automatic resolution from multiple sources. When accessing a configuration
    attribute (e.g., `Configuration.galileo_api_key`), the metaclass:

    1. Checks if the attribute is in _CONFIGURATION_KEYS
    2. Resolves the value from: explicit value → environment variable → .env file → default
    3. Applies type conversion and validation if a parser is defined
    4. Returns the resolved value

    When setting a configuration attribute (e.g., `Configuration.galileo_api_key = "key"`),
    the metaclass:

    1. Stores the value internally (with underscore prefix: `_galileo_api_key`)
    2. Automatically updates the corresponding environment variable
    3. Ensures compatibility with libraries that read from os.environ

    This pattern provides:
    - Extensibility: New configuration keys can be added to _CONFIGURATION_KEYS
    - Consistency: All configuration uses the same resolution pattern
    - Compatibility: Environment variables are kept in sync for third-party libraries
    - Transparency: Configuration sources are clearly prioritized

    Reference: https://docs.python.org/3/reference/datamodel.html#customizing-attribute-access
    """

    def __getattribute__(cls, name: str) -> Any:
        """
        Get configuration attribute value with automatic source resolution.

        Resolution order: explicit value → env var → .env file → default.

        Args:
            name: The attribute name to retrieve.

        Returns
        -------
            The resolved configuration value, or the attribute itself if not a config key.
        """
        if name.startswith("_") or name in (
            "connect",
            "reset",
            "is_configured",
            "get_configuration",
            "get_key_info",
            "get_keys_by_category",
        ):
            return super().__getattribute__(name)

        if name in _KEYS_BY_NAME:
            key = _KEYS_BY_NAME[name]
            super().__getattribute__("_load_env_file")()
            internal_name = f"_{name}"

            try:
                explicit_value = super().__getattribute__(internal_name)
                if explicit_value is not None:
                    return explicit_value
            except AttributeError:
                pass

            env_value = os.environ.get(key.env_var)
            if env_value is not None:
                if key.parser:
                    return key.parser(env_value)
                return env_value

            return key.default

        return super().__getattribute__(name)

    def __setattr__(cls, name: str, value: Any) -> None:
        """
        Set configuration attribute and sync to environment variable.

        When setting a configuration key, the value is:
        1. Stored internally with underscore prefix (e.g., `_galileo_api_key`)
        2. Synced to the corresponding environment variable (e.g., `GALILEO_API_KEY`)

        This ensures that both the Configuration class and environment variables
        remain consistent, maintaining compatibility with third-party libraries.

        Args:
            name: The attribute name to set.
            value: The value to assign to the attribute.
        """
        if name in _KEYS_BY_NAME:
            key = _KEYS_BY_NAME[name]
            internal_name = f"_{name}"
            super().__setattr__(internal_name, value)

            if value is not None:
                if key.value_type is bool:
                    os.environ[key.env_var] = str(value).lower()
                else:
                    os.environ[key.env_var] = str(value)
        else:
            super().__setattr__(name, value)


class Configuration(metaclass=ConfigurationMeta):
    """
    Single source of truth for SDK configuration.

    This class uses a metaclass pattern to provide dynamic attribute access to configuration
    keys defined in _CONFIGURATION_KEYS. Each configuration key can be accessed as a class
    attribute, with values resolved in the following priority order:

    1. Explicitly set value (via `Configuration.key = value`)
    2. Environment variable (e.g., GALILEO_API_KEY)
    3. .env file (loaded automatically on first access)
    4. Default value (defined in the key configuration)

    The metaclass automatically:
    - Syncs attribute assignments to environment variables
    - Loads .env files on first configuration access
    - Provides type conversion and validation via parsers

    Attributes
    ----------
    Configuration attributes are dynamically provided based on _CONFIGURATION_KEYS:
        galileo_api_key (str): API key for Galileo authentication (sensitive)
        console_url (str): URL of the Galileo console
        openai_api_key (str): OpenAI API key for SDK interoperability (sensitive)
        default_project_name (str): Default project name
        default_project_id (str): Default project ID
        default_logstream_name (str): Default log stream name
        default_logstream_id (str): Default log stream ID
        logging_disabled (bool): Disable all logging to Galileo

    Examples
    --------
    Reading configuration values:
    ```python
    # Access via class attribute (reads from env vars, .env, or defaults)
    api_key = Configuration.galileo_api_key
    url = Configuration.console_url
    ```

    Setting configuration values:
    ```python
    # Set explicitly (also updates environment variables)
    Configuration.galileo_api_key = "your-api-key"
    Configuration.console_url = "https://console.galileo.ai"
    ```

    Checking and connecting:
    ```python
    # Check if required configuration is present
    if Configuration.is_configured():
        Configuration.connect()
    ```

    Getting all configuration:
    ```python
    # Get all configuration values (sensitive values are masked)
    config = Configuration.get_configuration()
    print(config["galileo_api_key"])  # Output: "***"
    print(config["console_url"])       # Output: actual URL
    ```

    Resetting configuration:
    ```python
    # Clear all configuration values and environment variables
    Configuration.reset()
    ```

    Notes
    -----
    - The Configuration class should not be instantiated; use it as a static class
    - Direct attribute access (e.g., `Configuration.galileo_api_key`) is the recommended pattern
    - The `get_configuration()` method masks sensitive values for safe display/logging
    - Setting an attribute automatically updates the corresponding environment variable
    - This design maintains compatibility with third-party libraries expecting env vars
    """

    _env_loaded: bool = False

    @classmethod
    def _load_env_file(cls) -> None:
        """Load .env file if present. Called automatically on first access."""
        if cls._env_loaded:
            return

        env_file = Path(".env")
        if env_file.exists():
            try:
                with open(env_file) as f:
                    for line in f:
                        line = line.strip()
                        if not line or line.startswith("#"):
                            continue
                        if "=" in line:
                            key, value = line.split("=", 1)
                            key = key.strip()
                            value = value.strip().strip('"').strip("'")
                            if key not in os.environ:
                                os.environ[key] = value
            except Exception as e:
                logger.debug(f"Failed to parse .env file: {e}")

        cls._env_loaded = True

    @classmethod
    def connect(cls) -> None:
        """Validate configuration and connect to Galileo."""
        cls._load_env_file()

        if not cls.console_url:
            raise ConfigurationError(
                "Galileo console URL is required. Set Configuration.console_url or GALILEO_CONSOLE_URL."
            )

        if not cls.galileo_api_key:
            raise ConfigurationError(
                "Galileo API key is required. Set Configuration.galileo_api_key or GALILEO_API_KEY."
            )

        logger.info("Validating Galileo configuration and connectivity...")

        try:
            GalileoPythonConfig.get()
            logger.info("Successfully connected to Galileo")
        except Exception as e:
            error_msg = str(e)
            logger.error(f"Failed to connect to Galileo: {error_msg}")

            error_lower = error_msg.lower()
            if ("api" in error_lower and "key" in error_lower) or "auth" in error_lower:
                raise ConfigurationError(f"Authentication failed: {error_msg}") from e
            if "url" in error_lower or "connection" in error_lower:
                raise ConfigurationError(f"Connection failed: {error_msg}") from e
            raise ConfigurationError(f"Configuration validation failed: {error_msg}") from e

    @classmethod
    def reset(cls) -> None:
        """Reset all configuration values and clear environment variables."""
        for key in _CONFIGURATION_KEYS:
            with contextlib.suppress(AttributeError):
                delattr(cls, f"_{key.name}")
            if key.env_var in os.environ:
                del os.environ[key.env_var]

        cls._env_loaded = False

        try:
            if GalileoPythonConfig._instance is not None:
                GalileoPythonConfig._instance.reset()
        except Exception as e:
            logger.debug(f"Could not reset GalileoPythonConfig instance: {e}")

    @classmethod
    def is_configured(cls) -> bool:
        """Check if all required configuration keys are set."""
        cls._load_env_file()
        return all(getattr(cls, key.name) for key in _CONFIGURATION_KEYS if key.required)

    @classmethod
    def get_configuration(cls) -> dict[str, Any]:
        """Get all configuration values (sensitive values are masked)."""
        cls._load_env_file()
        result: dict[str, Any] = {}

        for key in _CONFIGURATION_KEYS:
            value = getattr(cls, key.name)
            if key.sensitive and value:
                result[key.name] = "***"
            elif value is None:
                result[key.name] = "Not set"
            else:
                result[key.name] = value

        result["is_configured"] = cls.is_configured()
        result["env_file_loaded"] = cls._env_loaded
        return result



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.project_bookmark_filter import ProjectBookmarkFilter
    from ..models.project_bookmark_sort import ProjectBookmarkSort
    from ..models.project_created_at_filter import ProjectCreatedAtFilter
    from ..models.project_created_at_sort_v1 import ProjectCreatedAtSortV1
    from ..models.project_creator_filter import ProjectCreatorFilter
    from ..models.project_id_filter import ProjectIDFilter
    from ..models.project_name_filter import ProjectNameFilter
    from ..models.project_name_sort_v1 import ProjectNameSortV1
    from ..models.project_runs_filter import ProjectRunsFilter
    from ..models.project_runs_sort import ProjectRunsSort
    from ..models.project_type_filter import ProjectTypeFilter
    from ..models.project_type_sort import ProjectTypeSort
    from ..models.project_updated_at_filter import ProjectUpdatedAtFilter
    from ..models.project_updated_at_sort_v1 import ProjectUpdatedAtSortV1


T = TypeVar("T", bound="ProjectCollectionParams")


@_attrs_define
class ProjectCollectionParams:
    """
    Attributes
    ----------
        filters (Union[Unset, list[Union['ProjectBookmarkFilter', 'ProjectCreatedAtFilter', 'ProjectCreatorFilter',
            'ProjectIDFilter', 'ProjectNameFilter', 'ProjectRunsFilter', 'ProjectTypeFilter', 'ProjectUpdatedAtFilter']]]):
        sort (Union['ProjectBookmarkSort', 'ProjectCreatedAtSortV1', 'ProjectNameSortV1', 'ProjectRunsSort',
            'ProjectTypeSort', 'ProjectUpdatedAtSortV1', None, Unset]):  Default: None.
    """

    filters: Union[
        Unset,
        list[
            Union[
                "ProjectBookmarkFilter",
                "ProjectCreatedAtFilter",
                "ProjectCreatorFilter",
                "ProjectIDFilter",
                "ProjectNameFilter",
                "ProjectRunsFilter",
                "ProjectTypeFilter",
                "ProjectUpdatedAtFilter",
            ]
        ],
    ] = UNSET
    sort: Union[
        "ProjectBookmarkSort",
        "ProjectCreatedAtSortV1",
        "ProjectNameSortV1",
        "ProjectRunsSort",
        "ProjectTypeSort",
        "ProjectUpdatedAtSortV1",
        None,
        Unset,
    ] = None
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.project_bookmark_sort import ProjectBookmarkSort
        from ..models.project_created_at_filter import ProjectCreatedAtFilter
        from ..models.project_created_at_sort_v1 import ProjectCreatedAtSortV1
        from ..models.project_creator_filter import ProjectCreatorFilter
        from ..models.project_id_filter import ProjectIDFilter
        from ..models.project_name_filter import ProjectNameFilter
        from ..models.project_name_sort_v1 import ProjectNameSortV1
        from ..models.project_runs_filter import ProjectRunsFilter
        from ..models.project_runs_sort import ProjectRunsSort
        from ..models.project_type_filter import ProjectTypeFilter
        from ..models.project_type_sort import ProjectTypeSort
        from ..models.project_updated_at_filter import ProjectUpdatedAtFilter
        from ..models.project_updated_at_sort_v1 import ProjectUpdatedAtSortV1

        filters: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item: dict[str, Any]
                if isinstance(
                    filters_item_data,
                    (
                        ProjectIDFilter,
                        ProjectNameFilter,
                        ProjectTypeFilter,
                        ProjectCreatorFilter,
                        ProjectCreatedAtFilter,
                        ProjectUpdatedAtFilter,
                        ProjectRunsFilter,
                    ),
                ):
                    filters_item = filters_item_data.to_dict()
                else:
                    filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        sort: Union[None, Unset, dict[str, Any]]
        if isinstance(self.sort, Unset):
            sort = UNSET
        elif isinstance(
            self.sort,
            (
                ProjectNameSortV1,
                ProjectTypeSort,
                ProjectCreatedAtSortV1,
                ProjectUpdatedAtSortV1,
                ProjectRunsSort,
                ProjectBookmarkSort,
            ),
        ):
            sort = self.sort.to_dict()
        else:
            sort = self.sort

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filters is not UNSET:
            field_dict["filters"] = filters
        if sort is not UNSET:
            field_dict["sort"] = sort

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.project_bookmark_filter import ProjectBookmarkFilter
        from ..models.project_bookmark_sort import ProjectBookmarkSort
        from ..models.project_created_at_filter import ProjectCreatedAtFilter
        from ..models.project_created_at_sort_v1 import ProjectCreatedAtSortV1
        from ..models.project_creator_filter import ProjectCreatorFilter
        from ..models.project_id_filter import ProjectIDFilter
        from ..models.project_name_filter import ProjectNameFilter
        from ..models.project_name_sort_v1 import ProjectNameSortV1
        from ..models.project_runs_filter import ProjectRunsFilter
        from ..models.project_runs_sort import ProjectRunsSort
        from ..models.project_type_filter import ProjectTypeFilter
        from ..models.project_type_sort import ProjectTypeSort
        from ..models.project_updated_at_filter import ProjectUpdatedAtFilter
        from ..models.project_updated_at_sort_v1 import ProjectUpdatedAtSortV1

        d = dict(src_dict)
        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:

            def _parse_filters_item(
                data: object,
            ) -> Union[
                "ProjectBookmarkFilter",
                "ProjectCreatedAtFilter",
                "ProjectCreatorFilter",
                "ProjectIDFilter",
                "ProjectNameFilter",
                "ProjectRunsFilter",
                "ProjectTypeFilter",
                "ProjectUpdatedAtFilter",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return ProjectIDFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return ProjectNameFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return ProjectTypeFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return ProjectCreatorFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return ProjectCreatedAtFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return ProjectUpdatedAtFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return ProjectRunsFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                return ProjectBookmarkFilter.from_dict(data)

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        def _parse_sort(
            data: object,
        ) -> Union[
            "ProjectBookmarkSort",
            "ProjectCreatedAtSortV1",
            "ProjectNameSortV1",
            "ProjectRunsSort",
            "ProjectTypeSort",
            "ProjectUpdatedAtSortV1",
            None,
            Unset,
        ]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ProjectNameSortV1.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ProjectTypeSort.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ProjectCreatedAtSortV1.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ProjectUpdatedAtSortV1.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ProjectRunsSort.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ProjectBookmarkSort.from_dict(data)

            except:  # noqa: E722
                pass
            return cast(
                Union[
                    "ProjectBookmarkSort",
                    "ProjectCreatedAtSortV1",
                    "ProjectNameSortV1",
                    "ProjectRunsSort",
                    "ProjectTypeSort",
                    "ProjectUpdatedAtSortV1",
                    None,
                    Unset,
                ],
                data,
            )

        sort = _parse_sort(d.pop("sort", UNSET))

        project_collection_params = cls(filters=filters, sort=sort)

        project_collection_params.additional_properties = d
        return project_collection_params

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs."""

from .action_result import ActionResult
from .action_type import ActionType
from .agent_span import AgentSpan
from .agent_span_dataset_metadata import AgentSpanDatasetMetadata
from .agent_span_user_metadata import AgentSpanUserMetadata
from .agent_type import AgentType
from .agentic_session_success_scorer import AgenticSessionSuccessScorer
from .agentic_session_success_scorer_type import AgenticSessionSuccessScorerType
from .agentic_session_success_template import AgenticSessionSuccessTemplate
from .agentic_session_success_template_response_schema_type_0 import AgenticSessionSuccessTemplateResponseSchemaType0
from .agentic_workflow_success_scorer import AgenticWorkflowSuccessScorer
from .agentic_workflow_success_scorer_type import AgenticWorkflowSuccessScorerType
from .agentic_workflow_success_template import AgenticWorkflowSuccessTemplate
from .agentic_workflow_success_template_response_schema_type_0 import AgenticWorkflowSuccessTemplateResponseSchemaType0
from .aggregated_trace_view_edge import AggregatedTraceViewEdge
from .aggregated_trace_view_graph import AggregatedTraceViewGraph
from .aggregated_trace_view_node import AggregatedTraceViewNode
from .aggregated_trace_view_node_metrics import AggregatedTraceViewNodeMetrics
from .aggregated_trace_view_request import AggregatedTraceViewRequest
from .aggregated_trace_view_response import AggregatedTraceViewResponse
from .and_node_log_records_filter import AndNodeLogRecordsFilter
from .anthropic_authentication_type import AnthropicAuthenticationType
from .anthropic_integration import AnthropicIntegration
from .anthropic_integration_create import AnthropicIntegrationCreate
from .anthropic_integration_create_custom_header_mapping_type_0 import (
    AnthropicIntegrationCreateCustomHeaderMappingType0,
)
from .anthropic_integration_custom_header_mapping_type_0 import AnthropicIntegrationCustomHeaderMappingType0
from .anthropic_integration_extra_type_0 import AnthropicIntegrationExtraType0
from .api_key_action import ApiKeyAction
from .api_key_login_request import ApiKeyLoginRequest
from .auth_method import AuthMethod
from .available_integrations import AvailableIntegrations
from .aws_bedrock_integration import AwsBedrockIntegration
from .aws_bedrock_integration_extra_type_0 import AwsBedrockIntegrationExtraType0
from .aws_bedrock_integration_inference_profiles import AwsBedrockIntegrationInferenceProfiles
from .aws_credential_type import AwsCredentialType
from .aws_sage_maker_integration import AwsSageMakerIntegration
from .aws_sage_maker_integration_create import AwsSageMakerIntegrationCreate
from .aws_sage_maker_integration_create_inference_profiles import AwsSageMakerIntegrationCreateInferenceProfiles
from .aws_sage_maker_integration_create_token import AwsSageMakerIntegrationCreateToken
from .aws_sage_maker_integration_extra_type_0 import AwsSageMakerIntegrationExtraType0
from .azure_authentication_type import AzureAuthenticationType
from .azure_integration import AzureIntegration
from .azure_integration_create import AzureIntegrationCreate
from .azure_integration_create_custom_header_mapping_type_0 import AzureIntegrationCreateCustomHeaderMappingType0
from .azure_integration_create_default_headers_type_0 import AzureIntegrationCreateDefaultHeadersType0
from .azure_integration_create_deployments import AzureIntegrationCreateDeployments
from .azure_integration_custom_header_mapping_type_0 import AzureIntegrationCustomHeaderMappingType0
from .azure_integration_default_headers_type_0 import AzureIntegrationDefaultHeadersType0
from .azure_integration_deployments import AzureIntegrationDeployments
from .azure_integration_extra_type_0 import AzureIntegrationExtraType0
from .azure_model_deployment import AzureModelDeployment
from .base_aws_integration_create import BaseAwsIntegrationCreate
from .base_aws_integration_create_inference_profiles import BaseAwsIntegrationCreateInferenceProfiles
from .base_aws_integration_create_token import BaseAwsIntegrationCreateToken
from .base_finetuned_scorer_db import BaseFinetunedScorerDB
from .base_finetuned_scorer_db_class_name_to_vocab_ix_type_0 import BaseFinetunedScorerDBClassNameToVocabIxType0
from .base_finetuned_scorer_db_class_name_to_vocab_ix_type_1 import BaseFinetunedScorerDBClassNameToVocabIxType1
from .base_generated_scorer_db import BaseGeneratedScorerDB
from .base_prompt_template_response import BasePromptTemplateResponse
from .base_prompt_template_version import BasePromptTemplateVersion
from .base_prompt_template_version_response import BasePromptTemplateVersionResponse
from .base_registered_scorer_db import BaseRegisteredScorerDB
from .base_scorer import BaseScorer
from .base_scorer_aggregates_type_0 import BaseScorerAggregatesType0
from .base_scorer_class_name_to_vocab_ix_type_0 import BaseScorerClassNameToVocabIxType0
from .base_scorer_class_name_to_vocab_ix_type_1 import BaseScorerClassNameToVocabIxType1
from .base_scorer_extra_type_0 import BaseScorerExtraType0
from .base_scorer_version_db import BaseScorerVersionDB
from .base_scorer_version_response import BaseScorerVersionResponse
from .bleu_scorer import BleuScorer
from .body_create_code_scorer_version_scorers_scorer_id_version_code_post import (
    BodyCreateCodeScorerVersionScorersScorerIdVersionCodePost,
)
from .body_create_dataset_datasets_post import BodyCreateDatasetDatasetsPost
from .body_login_email_login_post import BodyLoginEmailLoginPost
from .body_update_prompt_dataset_projects_project_id_prompt_datasets_dataset_id_put import (
    BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut,
)
from .body_upload_file_projects_project_id_upload_file_post import BodyUploadFileProjectsProjectIdUploadFilePost
from .body_upload_prompt_evaluation_dataset_projects_project_id_prompt_datasets_post import (
    BodyUploadPromptEvaluationDatasetProjectsProjectIdPromptDatasetsPost,
)
from .body_validate_code_scorer_scorers_code_validate_post import BodyValidateCodeScorerScorersCodeValidatePost
from .bucketed_metric import BucketedMetric
from .bucketed_metric_buckets import BucketedMetricBuckets
from .bucketed_metrics import BucketedMetrics
from .bulk_delete_datasets_request import BulkDeleteDatasetsRequest
from .bulk_delete_datasets_response import BulkDeleteDatasetsResponse
from .bulk_delete_failure import BulkDeleteFailure
from .bulk_delete_prompt_templates_request import BulkDeletePromptTemplatesRequest
from .chain_aggregation_strategy import ChainAggregationStrategy
from .chain_poll_template import ChainPollTemplate
from .chain_poll_template_response_schema_type_0 import ChainPollTemplateResponseSchemaType0
from .chunk_attribution_utilization_scorer import ChunkAttributionUtilizationScorer
from .chunk_attribution_utilization_scorer_type import ChunkAttributionUtilizationScorerType
from .chunk_attribution_utilization_template import ChunkAttributionUtilizationTemplate
from .chunk_attribution_utilization_template_response_schema_type_0 import (
    ChunkAttributionUtilizationTemplateResponseSchemaType0,
)
from .collaborator_role import CollaboratorRole
from .collaborator_role_info import CollaboratorRoleInfo
from .collaborator_update import CollaboratorUpdate
from .column_category import ColumnCategory
from .column_info import ColumnInfo
from .column_mapping import ColumnMapping
from .column_mapping_config import ColumnMappingConfig
from .completeness_scorer import CompletenessScorer
from .completeness_scorer_type import CompletenessScorerType
from .completeness_template import CompletenessTemplate
from .completeness_template_response_schema_type_0 import CompletenessTemplateResponseSchemaType0
from .context_adherence_scorer import ContextAdherenceScorer
from .context_adherence_scorer_type import ContextAdherenceScorerType
from .context_relevance_scorer import ContextRelevanceScorer
from .core_scorer_name import CoreScorerName
from .correctness_scorer import CorrectnessScorer
from .create_custom_luna_scorer_version_request import CreateCustomLunaScorerVersionRequest
from .create_job_request import CreateJobRequest
from .create_job_response import CreateJobResponse
from .create_llm_scorer_autogen_request import CreateLLMScorerAutogenRequest
from .create_llm_scorer_version_request import CreateLLMScorerVersionRequest
from .create_prompt_template_with_version_request_body import CreatePromptTemplateWithVersionRequestBody
from .create_scorer_request import CreateScorerRequest
from .create_scorer_version_request import CreateScorerVersionRequest
from .create_update_registered_scorer_response import CreateUpdateRegisteredScorerResponse
from .custom_authentication_type import CustomAuthenticationType
from .custom_integration import CustomIntegration
from .custom_integration_create import CustomIntegrationCreate
from .custom_integration_extra_type_0 import CustomIntegrationExtraType0
from .customized_agentic_session_success_gpt_scorer import CustomizedAgenticSessionSuccessGPTScorer
from .customized_agentic_session_success_gpt_scorer_aggregates_type_0 import (
    CustomizedAgenticSessionSuccessGPTScorerAggregatesType0,
)
from .customized_agentic_session_success_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedAgenticSessionSuccessGPTScorerClassNameToVocabIxType0,
)
from .customized_agentic_session_success_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedAgenticSessionSuccessGPTScorerClassNameToVocabIxType1,
)
from .customized_agentic_session_success_gpt_scorer_extra_type_0 import (
    CustomizedAgenticSessionSuccessGPTScorerExtraType0,
)
from .customized_agentic_workflow_success_gpt_scorer import CustomizedAgenticWorkflowSuccessGPTScorer
from .customized_agentic_workflow_success_gpt_scorer_aggregates_type_0 import (
    CustomizedAgenticWorkflowSuccessGPTScorerAggregatesType0,
)
from .customized_agentic_workflow_success_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedAgenticWorkflowSuccessGPTScorerClassNameToVocabIxType0,
)
from .customized_agentic_workflow_success_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedAgenticWorkflowSuccessGPTScorerClassNameToVocabIxType1,
)
from .customized_agentic_workflow_success_gpt_scorer_extra_type_0 import (
    CustomizedAgenticWorkflowSuccessGPTScorerExtraType0,
)
from .customized_chunk_attribution_utilization_gpt_scorer import CustomizedChunkAttributionUtilizationGPTScorer
from .customized_chunk_attribution_utilization_gpt_scorer_aggregates_type_0 import (
    CustomizedChunkAttributionUtilizationGPTScorerAggregatesType0,
)
from .customized_chunk_attribution_utilization_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedChunkAttributionUtilizationGPTScorerClassNameToVocabIxType0,
)
from .customized_chunk_attribution_utilization_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedChunkAttributionUtilizationGPTScorerClassNameToVocabIxType1,
)
from .customized_chunk_attribution_utilization_gpt_scorer_extra_type_0 import (
    CustomizedChunkAttributionUtilizationGPTScorerExtraType0,
)
from .customized_completeness_gpt_scorer import CustomizedCompletenessGPTScorer
from .customized_completeness_gpt_scorer_aggregates_type_0 import CustomizedCompletenessGPTScorerAggregatesType0
from .customized_completeness_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedCompletenessGPTScorerClassNameToVocabIxType0,
)
from .customized_completeness_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedCompletenessGPTScorerClassNameToVocabIxType1,
)
from .customized_completeness_gpt_scorer_extra_type_0 import CustomizedCompletenessGPTScorerExtraType0
from .customized_factuality_gpt_scorer import CustomizedFactualityGPTScorer
from .customized_factuality_gpt_scorer_aggregates_type_0 import CustomizedFactualityGPTScorerAggregatesType0
from .customized_factuality_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedFactualityGPTScorerClassNameToVocabIxType0,
)
from .customized_factuality_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedFactualityGPTScorerClassNameToVocabIxType1,
)
from .customized_factuality_gpt_scorer_extra_type_0 import CustomizedFactualityGPTScorerExtraType0
from .customized_ground_truth_adherence_gpt_scorer import CustomizedGroundTruthAdherenceGPTScorer
from .customized_ground_truth_adherence_gpt_scorer_aggregates_type_0 import (
    CustomizedGroundTruthAdherenceGPTScorerAggregatesType0,
)
from .customized_ground_truth_adherence_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedGroundTruthAdherenceGPTScorerClassNameToVocabIxType0,
)
from .customized_ground_truth_adherence_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedGroundTruthAdherenceGPTScorerClassNameToVocabIxType1,
)
from .customized_ground_truth_adherence_gpt_scorer_extra_type_0 import CustomizedGroundTruthAdherenceGPTScorerExtraType0
from .customized_groundedness_gpt_scorer import CustomizedGroundednessGPTScorer
from .customized_groundedness_gpt_scorer_aggregates_type_0 import CustomizedGroundednessGPTScorerAggregatesType0
from .customized_groundedness_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedGroundednessGPTScorerClassNameToVocabIxType0,
)
from .customized_groundedness_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedGroundednessGPTScorerClassNameToVocabIxType1,
)
from .customized_groundedness_gpt_scorer_extra_type_0 import CustomizedGroundednessGPTScorerExtraType0
from .customized_input_sexist_gpt_scorer import CustomizedInputSexistGPTScorer
from .customized_input_sexist_gpt_scorer_aggregates_type_0 import CustomizedInputSexistGPTScorerAggregatesType0
from .customized_input_sexist_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedInputSexistGPTScorerClassNameToVocabIxType0,
)
from .customized_input_sexist_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedInputSexistGPTScorerClassNameToVocabIxType1,
)
from .customized_input_sexist_gpt_scorer_extra_type_0 import CustomizedInputSexistGPTScorerExtraType0
from .customized_input_toxicity_gpt_scorer import CustomizedInputToxicityGPTScorer
from .customized_input_toxicity_gpt_scorer_aggregates_type_0 import CustomizedInputToxicityGPTScorerAggregatesType0
from .customized_input_toxicity_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedInputToxicityGPTScorerClassNameToVocabIxType0,
)
from .customized_input_toxicity_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedInputToxicityGPTScorerClassNameToVocabIxType1,
)
from .customized_input_toxicity_gpt_scorer_extra_type_0 import CustomizedInputToxicityGPTScorerExtraType0
from .customized_instruction_adherence_gpt_scorer import CustomizedInstructionAdherenceGPTScorer
from .customized_instruction_adherence_gpt_scorer_aggregates_type_0 import (
    CustomizedInstructionAdherenceGPTScorerAggregatesType0,
)
from .customized_instruction_adherence_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedInstructionAdherenceGPTScorerClassNameToVocabIxType0,
)
from .customized_instruction_adherence_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedInstructionAdherenceGPTScorerClassNameToVocabIxType1,
)
from .customized_instruction_adherence_gpt_scorer_extra_type_0 import CustomizedInstructionAdherenceGPTScorerExtraType0
from .customized_prompt_injection_gpt_scorer import CustomizedPromptInjectionGPTScorer
from .customized_prompt_injection_gpt_scorer_aggregates_type_0 import CustomizedPromptInjectionGPTScorerAggregatesType0
from .customized_prompt_injection_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedPromptInjectionGPTScorerClassNameToVocabIxType0,
)
from .customized_prompt_injection_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedPromptInjectionGPTScorerClassNameToVocabIxType1,
)
from .customized_prompt_injection_gpt_scorer_extra_type_0 import CustomizedPromptInjectionGPTScorerExtraType0
from .customized_sexist_gpt_scorer import CustomizedSexistGPTScorer
from .customized_sexist_gpt_scorer_aggregates_type_0 import CustomizedSexistGPTScorerAggregatesType0
from .customized_sexist_gpt_scorer_class_name_to_vocab_ix_type_0 import CustomizedSexistGPTScorerClassNameToVocabIxType0
from .customized_sexist_gpt_scorer_class_name_to_vocab_ix_type_1 import CustomizedSexistGPTScorerClassNameToVocabIxType1
from .customized_sexist_gpt_scorer_extra_type_0 import CustomizedSexistGPTScorerExtraType0
from .customized_tool_error_rate_gpt_scorer import CustomizedToolErrorRateGPTScorer
from .customized_tool_error_rate_gpt_scorer_aggregates_type_0 import CustomizedToolErrorRateGPTScorerAggregatesType0
from .customized_tool_error_rate_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedToolErrorRateGPTScorerClassNameToVocabIxType0,
)
from .customized_tool_error_rate_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedToolErrorRateGPTScorerClassNameToVocabIxType1,
)
from .customized_tool_error_rate_gpt_scorer_extra_type_0 import CustomizedToolErrorRateGPTScorerExtraType0
from .customized_tool_selection_quality_gpt_scorer import CustomizedToolSelectionQualityGPTScorer
from .customized_tool_selection_quality_gpt_scorer_aggregates_type_0 import (
    CustomizedToolSelectionQualityGPTScorerAggregatesType0,
)
from .customized_tool_selection_quality_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedToolSelectionQualityGPTScorerClassNameToVocabIxType0,
)
from .customized_tool_selection_quality_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedToolSelectionQualityGPTScorerClassNameToVocabIxType1,
)
from .customized_tool_selection_quality_gpt_scorer_extra_type_0 import CustomizedToolSelectionQualityGPTScorerExtraType0
from .customized_toxicity_gpt_scorer import CustomizedToxicityGPTScorer
from .customized_toxicity_gpt_scorer_aggregates_type_0 import CustomizedToxicityGPTScorerAggregatesType0
from .customized_toxicity_gpt_scorer_class_name_to_vocab_ix_type_0 import (
    CustomizedToxicityGPTScorerClassNameToVocabIxType0,
)
from .customized_toxicity_gpt_scorer_class_name_to_vocab_ix_type_1 import (
    CustomizedToxicityGPTScorerClassNameToVocabIxType1,
)
from .customized_toxicity_gpt_scorer_extra_type_0 import CustomizedToxicityGPTScorerExtraType0
from .data_type import DataType
from .data_type_options import DataTypeOptions
from .data_unit import DataUnit
from .databricks_edit_export_request import DatabricksEditExportRequest
from .databricks_edit_export_request_col_mapping_type_0 import DatabricksEditExportRequestColMappingType0
from .databricks_edit_export_response import DatabricksEditExportResponse
from .databricks_edit_export_response_col_mapping_type_0 import DatabricksEditExportResponseColMappingType0
from .databricks_export_request import DatabricksExportRequest
from .databricks_export_request_col_mapping_type_0 import DatabricksExportRequestColMappingType0
from .databricks_export_response import DatabricksExportResponse
from .databricks_export_response_col_mapping_type_0 import DatabricksExportResponseColMappingType0
from .databricks_integration import DatabricksIntegration
from .databricks_integration_create import DatabricksIntegrationCreate
from .databricks_integration_extra_type_0 import DatabricksIntegrationExtraType0
from .databricks_unity_catalog_edit_export_request import DatabricksUnityCatalogEditExportRequest
from .databricks_unity_catalog_edit_export_request_col_mapping_type_0 import (
    DatabricksUnityCatalogEditExportRequestColMappingType0,
)
from .databricks_unity_catalog_export_edits_response import DatabricksUnityCatalogExportEditsResponse
from .databricks_unity_catalog_export_edits_response_col_mapping_type_0 import (
    DatabricksUnityCatalogExportEditsResponseColMappingType0,
)
from .databricks_unity_catalog_export_response import DatabricksUnityCatalogExportResponse
from .databricks_unity_catalog_export_response_col_mapping_type_0 import (
    DatabricksUnityCatalogExportResponseColMappingType0,
)
from .dataset_action import DatasetAction
from .dataset_append_row import DatasetAppendRow
from .dataset_append_row_values import DatasetAppendRowValues
from .dataset_append_row_values_additional_property_type_3 import DatasetAppendRowValuesAdditionalPropertyType3
from .dataset_content import DatasetContent
from .dataset_content_filter import DatasetContentFilter
from .dataset_content_filter_operator import DatasetContentFilterOperator
from .dataset_content_sort_clause import DatasetContentSortClause
from .dataset_copy_record_data import DatasetCopyRecordData
from .dataset_created_at_sort import DatasetCreatedAtSort
from .dataset_data import DatasetData
from .dataset_db import DatasetDB
from .dataset_delete_row import DatasetDeleteRow
from .dataset_draft_filter import DatasetDraftFilter
from .dataset_draft_filter_operator import DatasetDraftFilterOperator
from .dataset_filter_rows import DatasetFilterRows
from .dataset_format import DatasetFormat
from .dataset_id_filter import DatasetIDFilter
from .dataset_id_filter_operator import DatasetIDFilterOperator
from .dataset_last_edited_by_user_at_sort import DatasetLastEditedByUserAtSort
from .dataset_name_filter import DatasetNameFilter
from .dataset_name_filter_operator import DatasetNameFilterOperator
from .dataset_name_sort import DatasetNameSort
from .dataset_not_in_project_filter import DatasetNotInProjectFilter
from .dataset_prepend_row import DatasetPrependRow
from .dataset_prepend_row_values import DatasetPrependRowValues
from .dataset_prepend_row_values_additional_property_type_3 import DatasetPrependRowValuesAdditionalPropertyType3
from .dataset_project import DatasetProject
from .dataset_project_last_used_at_sort import DatasetProjectLastUsedAtSort
from .dataset_projects_sort import DatasetProjectsSort
from .dataset_row import DatasetRow
from .dataset_row_metadata import DatasetRowMetadata
from .dataset_row_values_dict import DatasetRowValuesDict
from .dataset_row_values_dict_additional_property_type_3 import DatasetRowValuesDictAdditionalPropertyType3
from .dataset_row_values_item_type_3 import DatasetRowValuesItemType3
from .dataset_rows_sort import DatasetRowsSort
from .dataset_update_row import DatasetUpdateRow
from .dataset_update_row_values import DatasetUpdateRowValues
from .dataset_update_row_values_additional_property_type_3 import DatasetUpdateRowValuesAdditionalPropertyType3
from .dataset_updated_at_sort import DatasetUpdatedAtSort
from .dataset_used_in_project_filter import DatasetUsedInProjectFilter
from .dataset_version_db import DatasetVersionDB
from .dataset_version_index_sort import DatasetVersionIndexSort
from .delete_prompt_response import DeletePromptResponse
from .delete_run_response import DeleteRunResponse
from .delete_scorer_response import DeleteScorerResponse
from .document import Document
from .document_metadata import DocumentMetadata
from .edit_action import EditAction
from .edit_override import EditOverride
from .edit_override_edit_spans_type_0 import EditOverrideEditSpansType0
from .edit_span import EditSpan
from .event_status import EventStatus
from .execution_status import ExecutionStatus
from .experiment_create_request import ExperimentCreateRequest
from .experiment_dataset import ExperimentDataset
from .experiment_dataset_request import ExperimentDatasetRequest
from .experiment_metrics_request import ExperimentMetricsRequest
from .experiment_metrics_response import ExperimentMetricsResponse
from .experiment_phase_status import ExperimentPhaseStatus
from .experiment_playground import ExperimentPlayground
from .experiment_prompt import ExperimentPrompt
from .experiment_response import ExperimentResponse
from .experiment_response_aggregate_feedback import ExperimentResponseAggregateFeedback
from .experiment_response_aggregate_metrics import ExperimentResponseAggregateMetrics
from .experiment_response_rating_aggregates import ExperimentResponseRatingAggregates
from .experiment_response_rating_aggregates_additional_property import (
    ExperimentResponseRatingAggregatesAdditionalProperty,
)
from .experiment_response_tags import ExperimentResponseTags
from .experiment_status import ExperimentStatus
from .experiment_update_request import ExperimentUpdateRequest
from .experiments_available_columns_response import ExperimentsAvailableColumnsResponse
from .extended_agent_span_record import ExtendedAgentSpanRecord
from .extended_agent_span_record_dataset_metadata import ExtendedAgentSpanRecordDatasetMetadata
from .extended_agent_span_record_feedback_rating_info import ExtendedAgentSpanRecordFeedbackRatingInfo
from .extended_agent_span_record_metric_info_type_0 import ExtendedAgentSpanRecordMetricInfoType0
from .extended_agent_span_record_user_metadata import ExtendedAgentSpanRecordUserMetadata
from .extended_agent_span_record_with_children import ExtendedAgentSpanRecordWithChildren
from .extended_agent_span_record_with_children_dataset_metadata import (
    ExtendedAgentSpanRecordWithChildrenDatasetMetadata,
)
from .extended_agent_span_record_with_children_feedback_rating_info import (
    ExtendedAgentSpanRecordWithChildrenFeedbackRatingInfo,
)
from .extended_agent_span_record_with_children_metric_info_type_0 import (
    ExtendedAgentSpanRecordWithChildrenMetricInfoType0,
)
from .extended_agent_span_record_with_children_user_metadata import ExtendedAgentSpanRecordWithChildrenUserMetadata
from .extended_llm_span_record import ExtendedLlmSpanRecord
from .extended_llm_span_record_dataset_metadata import ExtendedLlmSpanRecordDatasetMetadata
from .extended_llm_span_record_feedback_rating_info import ExtendedLlmSpanRecordFeedbackRatingInfo
from .extended_llm_span_record_metric_info_type_0 import ExtendedLlmSpanRecordMetricInfoType0
from .extended_llm_span_record_tools_type_0_item import ExtendedLlmSpanRecordToolsType0Item
from .extended_llm_span_record_user_metadata import ExtendedLlmSpanRecordUserMetadata
from .extended_retriever_span_record import ExtendedRetrieverSpanRecord
from .extended_retriever_span_record_dataset_metadata import ExtendedRetrieverSpanRecordDatasetMetadata
from .extended_retriever_span_record_feedback_rating_info import ExtendedRetrieverSpanRecordFeedbackRatingInfo
from .extended_retriever_span_record_metric_info_type_0 import ExtendedRetrieverSpanRecordMetricInfoType0
from .extended_retriever_span_record_user_metadata import ExtendedRetrieverSpanRecordUserMetadata
from .extended_retriever_span_record_with_children import ExtendedRetrieverSpanRecordWithChildren
from .extended_retriever_span_record_with_children_dataset_metadata import (
    ExtendedRetrieverSpanRecordWithChildrenDatasetMetadata,
)
from .extended_retriever_span_record_with_children_feedback_rating_info import (
    ExtendedRetrieverSpanRecordWithChildrenFeedbackRatingInfo,
)
from .extended_retriever_span_record_with_children_metric_info_type_0 import (
    ExtendedRetrieverSpanRecordWithChildrenMetricInfoType0,
)
from .extended_retriever_span_record_with_children_user_metadata import (
    ExtendedRetrieverSpanRecordWithChildrenUserMetadata,
)
from .extended_session_record import ExtendedSessionRecord
from .extended_session_record_dataset_metadata import ExtendedSessionRecordDatasetMetadata
from .extended_session_record_feedback_rating_info import ExtendedSessionRecordFeedbackRatingInfo
from .extended_session_record_metric_info_type_0 import ExtendedSessionRecordMetricInfoType0
from .extended_session_record_user_metadata import ExtendedSessionRecordUserMetadata
from .extended_session_record_with_children import ExtendedSessionRecordWithChildren
from .extended_session_record_with_children_dataset_metadata import ExtendedSessionRecordWithChildrenDatasetMetadata
from .extended_session_record_with_children_feedback_rating_info import (
    ExtendedSessionRecordWithChildrenFeedbackRatingInfo,
)
from .extended_session_record_with_children_metric_info_type_0 import ExtendedSessionRecordWithChildrenMetricInfoType0
from .extended_session_record_with_children_user_metadata import ExtendedSessionRecordWithChildrenUserMetadata
from .extended_tool_span_record import ExtendedToolSpanRecord
from .extended_tool_span_record_dataset_metadata import ExtendedToolSpanRecordDatasetMetadata
from .extended_tool_span_record_feedback_rating_info import ExtendedToolSpanRecordFeedbackRatingInfo
from .extended_tool_span_record_metric_info_type_0 import ExtendedToolSpanRecordMetricInfoType0
from .extended_tool_span_record_user_metadata import ExtendedToolSpanRecordUserMetadata
from .extended_tool_span_record_with_children import ExtendedToolSpanRecordWithChildren
from .extended_tool_span_record_with_children_dataset_metadata import ExtendedToolSpanRecordWithChildrenDatasetMetadata
from .extended_tool_span_record_with_children_feedback_rating_info import (
    ExtendedToolSpanRecordWithChildrenFeedbackRatingInfo,
)
from .extended_tool_span_record_with_children_metric_info_type_0 import (
    ExtendedToolSpanRecordWithChildrenMetricInfoType0,
)
from .extended_tool_span_record_with_children_user_metadata import ExtendedToolSpanRecordWithChildrenUserMetadata
from .extended_trace_record import ExtendedTraceRecord
from .extended_trace_record_dataset_metadata import ExtendedTraceRecordDatasetMetadata
from .extended_trace_record_feedback_rating_info import ExtendedTraceRecordFeedbackRatingInfo
from .extended_trace_record_metric_info_type_0 import ExtendedTraceRecordMetricInfoType0
from .extended_trace_record_user_metadata import ExtendedTraceRecordUserMetadata
from .extended_trace_record_with_children import ExtendedTraceRecordWithChildren
from .extended_trace_record_with_children_dataset_metadata import ExtendedTraceRecordWithChildrenDatasetMetadata
from .extended_trace_record_with_children_feedback_rating_info import ExtendedTraceRecordWithChildrenFeedbackRatingInfo
from .extended_trace_record_with_children_metric_info_type_0 import ExtendedTraceRecordWithChildrenMetricInfoType0
from .extended_trace_record_with_children_user_metadata import ExtendedTraceRecordWithChildrenUserMetadata
from .extended_workflow_span_record import ExtendedWorkflowSpanRecord
from .extended_workflow_span_record_dataset_metadata import ExtendedWorkflowSpanRecordDatasetMetadata
from .extended_workflow_span_record_feedback_rating_info import ExtendedWorkflowSpanRecordFeedbackRatingInfo
from .extended_workflow_span_record_metric_info_type_0 import ExtendedWorkflowSpanRecordMetricInfoType0
from .extended_workflow_span_record_user_metadata import ExtendedWorkflowSpanRecordUserMetadata
from .extended_workflow_span_record_with_children import ExtendedWorkflowSpanRecordWithChildren
from .extended_workflow_span_record_with_children_dataset_metadata import (
    ExtendedWorkflowSpanRecordWithChildrenDatasetMetadata,
)
from .extended_workflow_span_record_with_children_feedback_rating_info import (
    ExtendedWorkflowSpanRecordWithChildrenFeedbackRatingInfo,
)
from .extended_workflow_span_record_with_children_metric_info_type_0 import (
    ExtendedWorkflowSpanRecordWithChildrenMetricInfoType0,
)
from .extended_workflow_span_record_with_children_user_metadata import (
    ExtendedWorkflowSpanRecordWithChildrenUserMetadata,
)
from .factuality_template import FactualityTemplate
from .factuality_template_response_schema_type_0 import FactualityTemplateResponseSchemaType0
from .feedback_aggregate import FeedbackAggregate
from .feedback_rating_db import FeedbackRatingDB
from .feedback_rating_info import FeedbackRatingInfo
from .feedback_type import FeedbackType
from .few_shot_example import FewShotExample
from .file_type import FileType
from .filter_leaf_log_records_filter import FilterLeafLogRecordsFilter
from .filter_params import FilterParams
from .fine_tuned_scorer import FineTunedScorer
from .fine_tuned_scorer_action import FineTunedScorerAction
from .fine_tuned_scorer_response import FineTunedScorerResponse
from .fine_tuned_scorer_response_class_name_to_vocab_ix_type_0 import FineTunedScorerResponseClassNameToVocabIxType0
from .fine_tuned_scorer_response_class_name_to_vocab_ix_type_1 import FineTunedScorerResponseClassNameToVocabIxType1
from .generated_scorer_action import GeneratedScorerAction
from .generated_scorer_configuration import GeneratedScorerConfiguration
from .generated_scorer_response import GeneratedScorerResponse
from .generated_scorer_validation_response import GeneratedScorerValidationResponse
from .generation_response import GenerationResponse
from .get_integration_status_integrations_name_status_get_response_get_integration_status_integrations_name_status_get import (
    GetIntegrationStatusIntegrationsNameStatusGetResponseGetIntegrationStatusIntegrationsNameStatusGet,
)
from .get_integrations_and_model_info_for_run_llm_integrations_projects_project_id_runs_run_id_get_get_run_integrations_response import (
    GetIntegrationsAndModelInfoForRunLlmIntegrationsProjectsProjectIdRunsRunIdGetGetRunIntegrationsResponse,
)
from .get_integrations_and_model_info_llm_integrations_get_response_get_integrations_and_model_info_llm_integrations_get import (
    GetIntegrationsAndModelInfoLlmIntegrationsGetResponseGetIntegrationsAndModelInfoLlmIntegrationsGet,
)
from .get_projects_paginated_response import GetProjectsPaginatedResponse
from .get_projects_paginated_response_v2 import GetProjectsPaginatedResponseV2
from .ground_truth_adherence_scorer import GroundTruthAdherenceScorer
from .ground_truth_adherence_template import GroundTruthAdherenceTemplate
from .ground_truth_adherence_template_response_schema_type_0 import GroundTruthAdherenceTemplateResponseSchemaType0
from .groundedness_template import GroundednessTemplate
from .groundedness_template_response_schema_type_0 import GroundednessTemplateResponseSchemaType0
from .group_action import GroupAction
from .group_collaborator import GroupCollaborator
from .group_collaborator_create import GroupCollaboratorCreate
from .group_member_action import GroupMemberAction
from .hallucination_segment import HallucinationSegment
from .healthcheck_response import HealthcheckResponse
from .histogram import Histogram
from .histogram_bucket import HistogramBucket
from .histogram_strategy import HistogramStrategy
from .http_validation_error import HTTPValidationError
from .image_generation_event import ImageGenerationEvent
from .image_generation_event_images_type_0_item import ImageGenerationEventImagesType0Item
from .image_generation_event_metadata_type_0 import ImageGenerationEventMetadataType0
from .input_map import InputMap
from .input_pii_scorer import InputPIIScorer
from .input_sexist_scorer import InputSexistScorer
from .input_sexist_scorer_type import InputSexistScorerType
from .input_sexist_template import InputSexistTemplate
from .input_sexist_template_response_schema_type_0 import InputSexistTemplateResponseSchemaType0
from .input_tone_scorer import InputToneScorer
from .input_toxicity_scorer import InputToxicityScorer
from .input_toxicity_scorer_type import InputToxicityScorerType
from .input_toxicity_template import InputToxicityTemplate
from .input_toxicity_template_response_schema_type_0 import InputToxicityTemplateResponseSchemaType0
from .input_type_enum import InputTypeEnum
from .insight_type import InsightType
from .instruction_adherence_scorer import InstructionAdherenceScorer
from .instruction_adherence_template import InstructionAdherenceTemplate
from .instruction_adherence_template_response_schema_type_0 import InstructionAdherenceTemplateResponseSchemaType0
from .integration_action import IntegrationAction
from .integration_db import IntegrationDB
from .integration_models_response import IntegrationModelsResponse
from .integration_name import IntegrationName
from .internal_tool_call import InternalToolCall
from .internal_tool_call_input_type_0 import InternalToolCallInputType0
from .internal_tool_call_metadata_type_0 import InternalToolCallMetadataType0
from .internal_tool_call_output_type_0 import InternalToolCallOutputType0
from .invalid_result import InvalidResult
from .invoke_response import InvokeResponse
from .invoke_response_headers_type_0 import InvokeResponseHeadersType0
from .invoke_response_metadata_type_0 import InvokeResponseMetadataType0
from .invoke_response_metric_results import InvokeResponseMetricResults
from .job_db import JobDB
from .job_db_request_data import JobDBRequestData
from .job_progress import JobProgress
from .label_studio_export_request import LabelStudioExportRequest
from .label_studio_integration import LabelStudioIntegration
from .label_studio_integration_create import LabelStudioIntegrationCreate
from .label_studio_integration_extra_type_0 import LabelStudioIntegrationExtraType0
from .label_studio_project import LabelStudioProject
from .lasso_selection import LassoSelection
from .like_dislike_aggregate import LikeDislikeAggregate
from .like_dislike_rating import LikeDislikeRating
from .list_dataset_params import ListDatasetParams
from .list_dataset_projects_response import ListDatasetProjectsResponse
from .list_dataset_response import ListDatasetResponse
from .list_dataset_version_params import ListDatasetVersionParams
from .list_dataset_version_response import ListDatasetVersionResponse
from .list_experiment_response import ListExperimentResponse
from .list_group_collaborators_response import ListGroupCollaboratorsResponse
from .list_log_stream_response import ListLogStreamResponse
from .list_prompt_dataset_response import ListPromptDatasetResponse
from .list_prompt_template_params import ListPromptTemplateParams
from .list_prompt_template_response import ListPromptTemplateResponse
from .list_prompt_template_version_params import ListPromptTemplateVersionParams
from .list_prompt_template_version_response import ListPromptTemplateVersionResponse
from .list_scorer_versions_response import ListScorerVersionsResponse
from .list_scorers_request import ListScorersRequest
from .list_scorers_response import ListScorersResponse
from .list_user_collaborators_response import ListUserCollaboratorsResponse
from .llm_export_format import LLMExportFormat
from .llm_integration import LLMIntegration
from .llm_metrics import LlmMetrics
from .llm_span import LlmSpan
from .llm_span_dataset_metadata import LlmSpanDatasetMetadata
from .llm_span_tools_type_0_item import LlmSpanToolsType0Item
from .llm_span_user_metadata import LlmSpanUserMetadata
from .log_records_available_columns_request import LogRecordsAvailableColumnsRequest
from .log_records_available_columns_response import LogRecordsAvailableColumnsResponse
from .log_records_boolean_filter import LogRecordsBooleanFilter
from .log_records_boolean_filter_operator import LogRecordsBooleanFilterOperator
from .log_records_collection_filter import LogRecordsCollectionFilter
from .log_records_collection_filter_operator import LogRecordsCollectionFilterOperator
from .log_records_column_info import LogRecordsColumnInfo
from .log_records_date_filter import LogRecordsDateFilter
from .log_records_date_filter_operator import LogRecordsDateFilterOperator
from .log_records_delete_request import LogRecordsDeleteRequest
from .log_records_delete_response import LogRecordsDeleteResponse
from .log_records_export_request import LogRecordsExportRequest
from .log_records_filter_type import LogRecordsFilterType
from .log_records_id_filter import LogRecordsIDFilter
from .log_records_id_filter_operator import LogRecordsIDFilterOperator
from .log_records_metrics_query_request import LogRecordsMetricsQueryRequest
from .log_records_metrics_response import LogRecordsMetricsResponse
from .log_records_metrics_response_aggregate_metrics import LogRecordsMetricsResponseAggregateMetrics
from .log_records_metrics_response_bucketed_metrics import LogRecordsMetricsResponseBucketedMetrics
from .log_records_number_filter import LogRecordsNumberFilter
from .log_records_number_filter_operator import LogRecordsNumberFilterOperator
from .log_records_query_count_request import LogRecordsQueryCountRequest
from .log_records_query_count_response import LogRecordsQueryCountResponse
from .log_records_query_request import LogRecordsQueryRequest
from .log_records_query_response import LogRecordsQueryResponse
from .log_records_sort_clause import LogRecordsSortClause
from .log_records_text_filter import LogRecordsTextFilter
from .log_records_text_filter_operator import LogRecordsTextFilterOperator
from .log_span_update_request import LogSpanUpdateRequest
from .log_span_update_response import LogSpanUpdateResponse
from .log_spans_ingest_request import LogSpansIngestRequest
from .log_spans_ingest_response import LogSpansIngestResponse
from .log_stream_create_request import LogStreamCreateRequest
from .log_stream_response import LogStreamResponse
from .log_stream_update_request import LogStreamUpdateRequest
from .log_trace_update_request import LogTraceUpdateRequest
from .log_trace_update_response import LogTraceUpdateResponse
from .log_traces_ingest_request import LogTracesIngestRequest
from .log_traces_ingest_response import LogTracesIngestResponse
from .logging_method import LoggingMethod
from .luna_input_type_enum import LunaInputTypeEnum
from .luna_output_type_enum import LunaOutputTypeEnum
from .manual_llm_validate_scorers_llm_validate_post_body import ManualLlmValidateScorersLlmValidatePostBody
from .mcp_approval_request_event import MCPApprovalRequestEvent
from .mcp_approval_request_event_metadata_type_0 import MCPApprovalRequestEventMetadataType0
from .mcp_approval_request_event_tool_invocation_type_0 import MCPApprovalRequestEventToolInvocationType0
from .mcp_call_event import MCPCallEvent
from .mcp_call_event_arguments_type_0 import MCPCallEventArgumentsType0
from .mcp_call_event_metadata_type_0 import MCPCallEventMetadataType0
from .mcp_call_event_result_type_0 import MCPCallEventResultType0
from .mcp_list_tools_event import MCPListToolsEvent
from .mcp_list_tools_event_metadata_type_0 import MCPListToolsEventMetadataType0
from .mcp_list_tools_event_tools_type_0_item import MCPListToolsEventToolsType0Item
from .message import Message
from .message_event import MessageEvent
from .message_event_content_parts_type_0_item import MessageEventContentPartsType0Item
from .message_event_metadata_type_0 import MessageEventMetadataType0
from .message_role import MessageRole
from .messages_list_item import MessagesListItem
from .messages_list_item_role import MessagesListItemRole
from .meta_filter import MetaFilter
from .metadata_filter import MetadataFilter
from .metadata_filter_operator import MetadataFilterOperator
from .metric_computation import MetricComputation
from .metric_computation_status import MetricComputationStatus
from .metric_computation_value_type_4 import MetricComputationValueType4
from .metric_computing import MetricComputing
from .metric_critique_columnar import MetricCritiqueColumnar
from .metric_critique_content import MetricCritiqueContent
from .metric_critique_job_configuration import MetricCritiqueJobConfiguration
from .metric_error import MetricError
from .metric_failed import MetricFailed
from .metric_not_applicable import MetricNotApplicable
from .metric_not_computed import MetricNotComputed
from .metric_pending import MetricPending
from .metric_settings_request import MetricSettingsRequest
from .metric_settings_response import MetricSettingsResponse
from .metric_success import MetricSuccess
from .metric_threshold import MetricThreshold
from .metrics import Metrics
from .metrics_testing_available_columns_request import MetricsTestingAvailableColumnsRequest
from .mistral_integration import MistralIntegration
from .mistral_integration_create import MistralIntegrationCreate
from .mistral_integration_extra_type_0 import MistralIntegrationExtraType0
from .model import Model
from .model_cost_by import ModelCostBy
from .model_type import ModelType
from .name import Name
from .node_name_filter import NodeNameFilter
from .node_name_filter_operator import NodeNameFilterOperator
from .node_type import NodeType
from .not_node_log_records_filter import NotNodeLogRecordsFilter
from .numeric_roll_up_method import NumericRollUpMethod
from .nvidia_integration import NvidiaIntegration
from .nvidia_integration_create import NvidiaIntegrationCreate
from .nvidia_integration_extra_type_0 import NvidiaIntegrationExtraType0
from .open_ai_function import OpenAIFunction
from .open_ai_integration import OpenAIIntegration
from .open_ai_integration_create import OpenAIIntegrationCreate
from .open_ai_integration_extra_type_0 import OpenAIIntegrationExtraType0
from .open_ai_tool_choice import OpenAIToolChoice
from .or_node_log_records_filter import OrNodeLogRecordsFilter
from .organization_action import OrganizationAction
from .output_map import OutputMap
from .output_pii_scorer import OutputPIIScorer
from .output_sexist_scorer import OutputSexistScorer
from .output_sexist_scorer_type import OutputSexistScorerType
from .output_tone_scorer import OutputToneScorer
from .output_toxicity_scorer import OutputToxicityScorer
from .output_toxicity_scorer_type import OutputToxicityScorerType
from .output_type_enum import OutputTypeEnum
from .override_action import OverrideAction
from .passthrough_action import PassthroughAction
from .payload import Payload
from .permission import Permission
from .preview_dataset_request import PreviewDatasetRequest
from .project_action import ProjectAction
from .project_bookmark_filter import ProjectBookmarkFilter
from .project_bookmark_sort import ProjectBookmarkSort
from .project_collection_params import ProjectCollectionParams
from .project_create import ProjectCreate
from .project_create_response import ProjectCreateResponse
from .project_created_at_filter import ProjectCreatedAtFilter
from .project_created_at_filter_operator import ProjectCreatedAtFilterOperator
from .project_created_at_sort_v1 import ProjectCreatedAtSortV1
from .project_creator_filter import ProjectCreatorFilter
from .project_creator_filter_operator import ProjectCreatorFilterOperator
from .project_db import ProjectDB
from .project_db_thin import ProjectDBThin
from .project_delete_response import ProjectDeleteResponse
from .project_id_filter import ProjectIDFilter
from .project_id_filter_operator import ProjectIDFilterOperator
from .project_item import ProjectItem
from .project_labels import ProjectLabels
from .project_name_filter import ProjectNameFilter
from .project_name_filter_operator import ProjectNameFilterOperator
from .project_name_sort_v1 import ProjectNameSortV1
from .project_runs_filter import ProjectRunsFilter
from .project_runs_filter_operator import ProjectRunsFilterOperator
from .project_runs_sort import ProjectRunsSort
from .project_type import ProjectType
from .project_type_filter import ProjectTypeFilter
from .project_type_filter_operator import ProjectTypeFilterOperator
from .project_type_sort import ProjectTypeSort
from .project_update import ProjectUpdate
from .project_update_response import ProjectUpdateResponse
from .project_updated_at_filter import ProjectUpdatedAtFilter
from .project_updated_at_filter_operator import ProjectUpdatedAtFilterOperator
from .project_updated_at_sort_v1 import ProjectUpdatedAtSortV1
from .prompt_dataset_db import PromptDatasetDB
from .prompt_injection_scorer import PromptInjectionScorer
from .prompt_injection_scorer_type import PromptInjectionScorerType
from .prompt_injection_template import PromptInjectionTemplate
from .prompt_injection_template_response_schema_type_0 import PromptInjectionTemplateResponseSchemaType0
from .prompt_optimization_configuration import PromptOptimizationConfiguration
from .prompt_perplexity_scorer import PromptPerplexityScorer
from .prompt_run_settings import PromptRunSettings
from .prompt_run_settings_response_format_type_0 import PromptRunSettingsResponseFormatType0
from .prompt_run_settings_tools_type_0_item import PromptRunSettingsToolsType0Item
from .prompt_template_created_at_sort import PromptTemplateCreatedAtSort
from .prompt_template_created_by_filter import PromptTemplateCreatedByFilter
from .prompt_template_created_by_filter_operator import PromptTemplateCreatedByFilterOperator
from .prompt_template_name_filter import PromptTemplateNameFilter
from .prompt_template_name_filter_operator import PromptTemplateNameFilterOperator
from .prompt_template_name_sort import PromptTemplateNameSort
from .prompt_template_not_in_project_filter import PromptTemplateNotInProjectFilter
from .prompt_template_updated_at_sort import PromptTemplateUpdatedAtSort
from .prompt_template_used_in_project_filter import PromptTemplateUsedInProjectFilter
from .prompt_template_version_created_at_sort import PromptTemplateVersionCreatedAtSort
from .prompt_template_version_number_sort import PromptTemplateVersionNumberSort
from .prompt_template_version_updated_at_sort import PromptTemplateVersionUpdatedAtSort
from .protect_request import ProtectRequest
from .protect_request_headers_type_0 import ProtectRequestHeadersType0
from .protect_request_metadata_type_0 import ProtectRequestMetadataType0
from .protect_response import ProtectResponse
from .query_dataset_params import QueryDatasetParams
from .reasoning_event import ReasoningEvent
from .reasoning_event_metadata_type_0 import ReasoningEventMetadataType0
from .recompute_log_records_metrics_request import RecomputeLogRecordsMetricsRequest
from .recompute_settings_log_stream import RecomputeSettingsLogStream
from .recompute_settings_observe import RecomputeSettingsObserve
from .recompute_settings_project import RecomputeSettingsProject
from .recompute_settings_runs import RecomputeSettingsRuns
from .registered_scorer import RegisteredScorer
from .registered_scorer_action import RegisteredScorerAction
from .registered_scorer_task_result_response import RegisteredScorerTaskResultResponse
from .render_template_request import RenderTemplateRequest
from .render_template_response import RenderTemplateResponse
from .rendered_template import RenderedTemplate
from .retriever_span import RetrieverSpan
from .retriever_span_dataset_metadata import RetrieverSpanDatasetMetadata
from .retriever_span_user_metadata import RetrieverSpanUserMetadata
from .roll_up_strategy import RollUpStrategy
from .rollback_request import RollbackRequest
from .root_type import RootType
from .rouge_scorer import RougeScorer
from .rule import Rule
from .rule_operator import RuleOperator
from .rule_result import RuleResult
from .ruleset import Ruleset
from .ruleset_result import RulesetResult
from .rulesets_mixin import RulesetsMixin
from .run_created_at_filter import RunCreatedAtFilter
from .run_created_at_filter_operator import RunCreatedAtFilterOperator
from .run_created_at_sort import RunCreatedAtSort
from .run_created_by_filter import RunCreatedByFilter
from .run_created_by_filter_operator import RunCreatedByFilterOperator
from .run_db import RunDB
from .run_db_thin import RunDBThin
from .run_id_filter import RunIDFilter
from .run_id_filter_operator import RunIDFilterOperator
from .run_name_filter import RunNameFilter
from .run_name_filter_operator import RunNameFilterOperator
from .run_name_sort import RunNameSort
from .run_params_map import RunParamsMap
from .run_scorer_settings_patch_request import RunScorerSettingsPatchRequest
from .run_scorer_settings_response import RunScorerSettingsResponse
from .run_tag_create_request import RunTagCreateRequest
from .run_tag_db import RunTagDB
from .run_updated_at_filter import RunUpdatedAtFilter
from .run_updated_at_filter_operator import RunUpdatedAtFilterOperator
from .run_updated_at_sort import RunUpdatedAtSort
from .score_aggregate import ScoreAggregate
from .score_rating import ScoreRating
from .scorer_config import ScorerConfig
from .scorer_created_at_filter import ScorerCreatedAtFilter
from .scorer_created_at_filter_operator import ScorerCreatedAtFilterOperator
from .scorer_creator_filter import ScorerCreatorFilter
from .scorer_creator_filter_operator import ScorerCreatorFilterOperator
from .scorer_defaults import ScorerDefaults
from .scorer_enabled_in_run_sort import ScorerEnabledInRunSort
from .scorer_label_filter import ScorerLabelFilter
from .scorer_label_filter_operator import ScorerLabelFilterOperator
from .scorer_model_type_filter import ScorerModelTypeFilter
from .scorer_name import ScorerName
from .scorer_name_filter import ScorerNameFilter
from .scorer_name_filter_operator import ScorerNameFilterOperator
from .scorer_name_sort import ScorerNameSort
from .scorer_response import ScorerResponse
from .scorer_tags_filter import ScorerTagsFilter
from .scorer_tags_filter_operator import ScorerTagsFilterOperator
from .scorer_type import ScorerType
from .scorer_type_filter import ScorerTypeFilter
from .scorer_type_filter_operator import ScorerTypeFilterOperator
from .scorer_types import ScorerTypes
from .scorer_updated_at_filter import ScorerUpdatedAtFilter
from .scorer_updated_at_filter_operator import ScorerUpdatedAtFilterOperator
from .scorers_configuration import ScorersConfiguration
from .segment import Segment
from .segment_filter import SegmentFilter
from .session_create_request import SessionCreateRequest
from .session_create_response import SessionCreateResponse
from .sexist_template import SexistTemplate
from .sexist_template_response_schema_type_0 import SexistTemplateResponseSchemaType0
from .split import Split
from .stage_db import StageDB
from .stage_metadata import StageMetadata
from .stage_type import StageType
from .stage_with_rulesets import StageWithRulesets
from .star_aggregate import StarAggregate
from .star_aggregate_counts import StarAggregateCounts
from .star_rating import StarRating
from .step_type import StepType
from .string_data import StringData
from .subscription_config import SubscriptionConfig
from .synthetic_data_source_dataset import SyntheticDataSourceDataset
from .synthetic_data_types import SyntheticDataTypes
from .synthetic_dataset_extension_request import SyntheticDatasetExtensionRequest
from .synthetic_dataset_extension_response import SyntheticDatasetExtensionResponse
from .system_metric_info import SystemMetricInfo
from .tagging_schema import TaggingSchema
from .tags_aggregate import TagsAggregate
from .tags_aggregate_counts import TagsAggregateCounts
from .tags_rating import TagsRating
from .task_resource_limits import TaskResourceLimits
from .task_result_status import TaskResultStatus
from .task_type import TaskType
from .template_stub_request import TemplateStubRequest
from .test_score import TestScore
from .text_aggregate import TextAggregate
from .text_rating import TextRating
from .token import Token
from .tool_call import ToolCall
from .tool_call_function import ToolCallFunction
from .tool_error_rate_scorer import ToolErrorRateScorer
from .tool_error_rate_scorer_type import ToolErrorRateScorerType
from .tool_error_rate_template import ToolErrorRateTemplate
from .tool_error_rate_template_response_schema_type_0 import ToolErrorRateTemplateResponseSchemaType0
from .tool_selection_quality_scorer import ToolSelectionQualityScorer
from .tool_selection_quality_scorer_type import ToolSelectionQualityScorerType
from .tool_selection_quality_template import ToolSelectionQualityTemplate
from .tool_selection_quality_template_response_schema_type_0 import ToolSelectionQualityTemplateResponseSchemaType0
from .tool_span import ToolSpan
from .tool_span_dataset_metadata import ToolSpanDatasetMetadata
from .tool_span_user_metadata import ToolSpanUserMetadata
from .toxicity_template import ToxicityTemplate
from .toxicity_template_response_schema_type_0 import ToxicityTemplateResponseSchemaType0
from .trace import Trace
from .trace_dataset_metadata import TraceDatasetMetadata
from .trace_metadata import TraceMetadata
from .trace_user_metadata import TraceUserMetadata
from .uncertainty_scorer import UncertaintyScorer
from .update_dataset_content_request import UpdateDatasetContentRequest
from .update_dataset_request import UpdateDatasetRequest
from .update_dataset_version_request import UpdateDatasetVersionRequest
from .update_prompt_template_request import UpdatePromptTemplateRequest
from .update_scorer_request import UpdateScorerRequest
from .upsert_dataset_content_request import UpsertDatasetContentRequest
from .user_action import UserAction
from .user_collaborator import UserCollaborator
from .user_collaborator_create import UserCollaboratorCreate
from .user_db import UserDB
from .user_info import UserInfo
from .user_role import UserRole
from .valid_result import ValidResult
from .validate_code_scorer_response import ValidateCodeScorerResponse
from .validate_llm_scorer_log_record_request import ValidateLLMScorerLogRecordRequest
from .validate_llm_scorer_log_record_response import ValidateLLMScorerLogRecordResponse
from .validate_registered_scorer_result import ValidateRegisteredScorerResult
from .validation_error import ValidationError
from .vegas_gateway_integration import VegasGatewayIntegration
from .vegas_gateway_integration_create import VegasGatewayIntegrationCreate
from .vegas_gateway_integration_extra_type_0 import VegasGatewayIntegrationExtraType0
from .vertex_ai_integration import VertexAIIntegration
from .vertex_ai_integration_create import VertexAIIntegrationCreate
from .vertex_ai_integration_extra_type_0 import VertexAIIntegrationExtraType0
from .workflow_span import WorkflowSpan
from .workflow_span_dataset_metadata import WorkflowSpanDatasetMetadata
from .workflow_span_user_metadata import WorkflowSpanUserMetadata
from .writer_integration import WriterIntegration
from .writer_integration_create import WriterIntegrationCreate
from .writer_integration_extra_type_0 import WriterIntegrationExtraType0

__all__ = (
    "ActionResult",
    "ActionType",
    "AgentSpan",
    "AgentSpanDatasetMetadata",
    "AgentSpanUserMetadata",
    "AgentType",
    "AgenticSessionSuccessScorer",
    "AgenticSessionSuccessScorerType",
    "AgenticSessionSuccessTemplate",
    "AgenticSessionSuccessTemplateResponseSchemaType0",
    "AgenticWorkflowSuccessScorer",
    "AgenticWorkflowSuccessScorerType",
    "AgenticWorkflowSuccessTemplate",
    "AgenticWorkflowSuccessTemplateResponseSchemaType0",
    "AggregatedTraceViewEdge",
    "AggregatedTraceViewGraph",
    "AggregatedTraceViewNode",
    "AggregatedTraceViewNodeMetrics",
    "AggregatedTraceViewRequest",
    "AggregatedTraceViewResponse",
    "AndNodeLogRecordsFilter",
    "AnthropicAuthenticationType",
    "AnthropicIntegration",
    "AnthropicIntegrationCreate",
    "AnthropicIntegrationCreateCustomHeaderMappingType0",
    "AnthropicIntegrationCustomHeaderMappingType0",
    "AnthropicIntegrationExtraType0",
    "ApiKeyAction",
    "ApiKeyLoginRequest",
    "AuthMethod",
    "AvailableIntegrations",
    "AwsBedrockIntegration",
    "AwsBedrockIntegrationExtraType0",
    "AwsBedrockIntegrationInferenceProfiles",
    "AwsCredentialType",
    "AwsSageMakerIntegration",
    "AwsSageMakerIntegrationCreate",
    "AwsSageMakerIntegrationCreateInferenceProfiles",
    "AwsSageMakerIntegrationCreateToken",
    "AwsSageMakerIntegrationExtraType0",
    "AzureAuthenticationType",
    "AzureIntegration",
    "AzureIntegrationCreate",
    "AzureIntegrationCreateCustomHeaderMappingType0",
    "AzureIntegrationCreateDefaultHeadersType0",
    "AzureIntegrationCreateDeployments",
    "AzureIntegrationCustomHeaderMappingType0",
    "AzureIntegrationDefaultHeadersType0",
    "AzureIntegrationDeployments",
    "AzureIntegrationExtraType0",
    "AzureModelDeployment",
    "BaseAwsIntegrationCreate",
    "BaseAwsIntegrationCreateInferenceProfiles",
    "BaseAwsIntegrationCreateToken",
    "BaseFinetunedScorerDB",
    "BaseFinetunedScorerDBClassNameToVocabIxType0",
    "BaseFinetunedScorerDBClassNameToVocabIxType1",
    "BaseGeneratedScorerDB",
    "BasePromptTemplateResponse",
    "BasePromptTemplateVersion",
    "BasePromptTemplateVersionResponse",
    "BaseRegisteredScorerDB",
    "BaseScorer",
    "BaseScorerAggregatesType0",
    "BaseScorerClassNameToVocabIxType0",
    "BaseScorerClassNameToVocabIxType1",
    "BaseScorerExtraType0",
    "BaseScorerVersionDB",
    "BaseScorerVersionResponse",
    "BleuScorer",
    "BodyCreateCodeScorerVersionScorersScorerIdVersionCodePost",
    "BodyCreateDatasetDatasetsPost",
    "BodyLoginEmailLoginPost",
    "BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut",
    "BodyUploadFileProjectsProjectIdUploadFilePost",
    "BodyUploadPromptEvaluationDatasetProjectsProjectIdPromptDatasetsPost",
    "BodyValidateCodeScorerScorersCodeValidatePost",
    "BucketedMetric",
    "BucketedMetricBuckets",
    "BucketedMetrics",
    "BulkDeleteDatasetsRequest",
    "BulkDeleteDatasetsResponse",
    "BulkDeleteFailure",
    "BulkDeletePromptTemplatesRequest",
    "ChainAggregationStrategy",
    "ChainPollTemplate",
    "ChainPollTemplateResponseSchemaType0",
    "ChunkAttributionUtilizationScorer",
    "ChunkAttributionUtilizationScorerType",
    "ChunkAttributionUtilizationTemplate",
    "ChunkAttributionUtilizationTemplateResponseSchemaType0",
    "CollaboratorRole",
    "CollaboratorRoleInfo",
    "CollaboratorUpdate",
    "ColumnCategory",
    "ColumnInfo",
    "ColumnMapping",
    "ColumnMappingConfig",
    "CompletenessScorer",
    "CompletenessScorerType",
    "CompletenessTemplate",
    "CompletenessTemplateResponseSchemaType0",
    "ContextAdherenceScorer",
    "ContextAdherenceScorerType",
    "ContextRelevanceScorer",
    "CoreScorerName",
    "CorrectnessScorer",
    "CreateCustomLunaScorerVersionRequest",
    "CreateJobRequest",
    "CreateJobResponse",
    "CreateLLMScorerAutogenRequest",
    "CreateLLMScorerVersionRequest",
    "CreatePromptTemplateWithVersionRequestBody",
    "CreateScorerRequest",
    "CreateScorerVersionRequest",
    "CreateUpdateRegisteredScorerResponse",
    "CustomAuthenticationType",
    "CustomIntegration",
    "CustomIntegrationCreate",
    "CustomIntegrationExtraType0",
    "CustomizedAgenticSessionSuccessGPTScorer",
    "CustomizedAgenticSessionSuccessGPTScorerAggregatesType0",
    "CustomizedAgenticSessionSuccessGPTScorerClassNameToVocabIxType0",
    "CustomizedAgenticSessionSuccessGPTScorerClassNameToVocabIxType1",
    "CustomizedAgenticSessionSuccessGPTScorerExtraType0",
    "CustomizedAgenticWorkflowSuccessGPTScorer",
    "CustomizedAgenticWorkflowSuccessGPTScorerAggregatesType0",
    "CustomizedAgenticWorkflowSuccessGPTScorerClassNameToVocabIxType0",
    "CustomizedAgenticWorkflowSuccessGPTScorerClassNameToVocabIxType1",
    "CustomizedAgenticWorkflowSuccessGPTScorerExtraType0",
    "CustomizedChunkAttributionUtilizationGPTScorer",
    "CustomizedChunkAttributionUtilizationGPTScorerAggregatesType0",
    "CustomizedChunkAttributionUtilizationGPTScorerClassNameToVocabIxType0",
    "CustomizedChunkAttributionUtilizationGPTScorerClassNameToVocabIxType1",
    "CustomizedChunkAttributionUtilizationGPTScorerExtraType0",
    "CustomizedCompletenessGPTScorer",
    "CustomizedCompletenessGPTScorerAggregatesType0",
    "CustomizedCompletenessGPTScorerClassNameToVocabIxType0",
    "CustomizedCompletenessGPTScorerClassNameToVocabIxType1",
    "CustomizedCompletenessGPTScorerExtraType0",
    "CustomizedFactualityGPTScorer",
    "CustomizedFactualityGPTScorerAggregatesType0",
    "CustomizedFactualityGPTScorerClassNameToVocabIxType0",
    "CustomizedFactualityGPTScorerClassNameToVocabIxType1",
    "CustomizedFactualityGPTScorerExtraType0",
    "CustomizedGroundTruthAdherenceGPTScorer",
    "CustomizedGroundTruthAdherenceGPTScorerAggregatesType0",
    "CustomizedGroundTruthAdherenceGPTScorerClassNameToVocabIxType0",
    "CustomizedGroundTruthAdherenceGPTScorerClassNameToVocabIxType1",
    "CustomizedGroundTruthAdherenceGPTScorerExtraType0",
    "CustomizedGroundednessGPTScorer",
    "CustomizedGroundednessGPTScorerAggregatesType0",
    "CustomizedGroundednessGPTScorerClassNameToVocabIxType0",
    "CustomizedGroundednessGPTScorerClassNameToVocabIxType1",
    "CustomizedGroundednessGPTScorerExtraType0",
    "CustomizedInputSexistGPTScorer",
    "CustomizedInputSexistGPTScorerAggregatesType0",
    "CustomizedInputSexistGPTScorerClassNameToVocabIxType0",
    "CustomizedInputSexistGPTScorerClassNameToVocabIxType1",
    "CustomizedInputSexistGPTScorerExtraType0",
    "CustomizedInputToxicityGPTScorer",
    "CustomizedInputToxicityGPTScorerAggregatesType0",
    "CustomizedInputToxicityGPTScorerClassNameToVocabIxType0",
    "CustomizedInputToxicityGPTScorerClassNameToVocabIxType1",
    "CustomizedInputToxicityGPTScorerExtraType0",
    "CustomizedInstructionAdherenceGPTScorer",
    "CustomizedInstructionAdherenceGPTScorerAggregatesType0",
    "CustomizedInstructionAdherenceGPTScorerClassNameToVocabIxType0",
    "CustomizedInstructionAdherenceGPTScorerClassNameToVocabIxType1",
    "CustomizedInstructionAdherenceGPTScorerExtraType0",
    "CustomizedPromptInjectionGPTScorer",
    "CustomizedPromptInjectionGPTScorerAggregatesType0",
    "CustomizedPromptInjectionGPTScorerClassNameToVocabIxType0",
    "CustomizedPromptInjectionGPTScorerClassNameToVocabIxType1",
    "CustomizedPromptInjectionGPTScorerExtraType0",
    "CustomizedSexistGPTScorer",
    "CustomizedSexistGPTScorerAggregatesType0",
    "CustomizedSexistGPTScorerClassNameToVocabIxType0",
    "CustomizedSexistGPTScorerClassNameToVocabIxType1",
    "CustomizedSexistGPTScorerExtraType0",
    "CustomizedToolErrorRateGPTScorer",
    "CustomizedToolErrorRateGPTScorerAggregatesType0",
    "CustomizedToolErrorRateGPTScorerClassNameToVocabIxType0",
    "CustomizedToolErrorRateGPTScorerClassNameToVocabIxType1",
    "CustomizedToolErrorRateGPTScorerExtraType0",
    "CustomizedToolSelectionQualityGPTScorer",
    "CustomizedToolSelectionQualityGPTScorerAggregatesType0",
    "CustomizedToolSelectionQualityGPTScorerClassNameToVocabIxType0",
    "CustomizedToolSelectionQualityGPTScorerClassNameToVocabIxType1",
    "CustomizedToolSelectionQualityGPTScorerExtraType0",
    "CustomizedToxicityGPTScorer",
    "CustomizedToxicityGPTScorerAggregatesType0",
    "CustomizedToxicityGPTScorerClassNameToVocabIxType0",
    "CustomizedToxicityGPTScorerClassNameToVocabIxType1",
    "CustomizedToxicityGPTScorerExtraType0",
    "DataType",
    "DataTypeOptions",
    "DataUnit",
    "DatabricksEditExportRequest",
    "DatabricksEditExportRequestColMappingType0",
    "DatabricksEditExportResponse",
    "DatabricksEditExportResponseColMappingType0",
    "DatabricksExportRequest",
    "DatabricksExportRequestColMappingType0",
    "DatabricksExportResponse",
    "DatabricksExportResponseColMappingType0",
    "DatabricksIntegration",
    "DatabricksIntegrationCreate",
    "DatabricksIntegrationExtraType0",
    "DatabricksUnityCatalogEditExportRequest",
    "DatabricksUnityCatalogEditExportRequestColMappingType0",
    "DatabricksUnityCatalogExportEditsResponse",
    "DatabricksUnityCatalogExportEditsResponseColMappingType0",
    "DatabricksUnityCatalogExportResponse",
    "DatabricksUnityCatalogExportResponseColMappingType0",
    "DatasetAction",
    "DatasetAppendRow",
    "DatasetAppendRowValues",
    "DatasetAppendRowValuesAdditionalPropertyType3",
    "DatasetContent",
    "DatasetContentFilter",
    "DatasetContentFilterOperator",
    "DatasetContentSortClause",
    "DatasetCopyRecordData",
    "DatasetCreatedAtSort",
    "DatasetDB",
    "DatasetData",
    "DatasetDeleteRow",
    "DatasetDraftFilter",
    "DatasetDraftFilterOperator",
    "DatasetFilterRows",
    "DatasetFormat",
    "DatasetIDFilter",
    "DatasetIDFilterOperator",
    "DatasetLastEditedByUserAtSort",
    "DatasetNameFilter",
    "DatasetNameFilterOperator",
    "DatasetNameSort",
    "DatasetNotInProjectFilter",
    "DatasetPrependRow",
    "DatasetPrependRowValues",
    "DatasetPrependRowValuesAdditionalPropertyType3",
    "DatasetProject",
    "DatasetProjectLastUsedAtSort",
    "DatasetProjectsSort",
    "DatasetRow",
    "DatasetRowMetadata",
    "DatasetRowValuesDict",
    "DatasetRowValuesDictAdditionalPropertyType3",
    "DatasetRowValuesItemType3",
    "DatasetRowsSort",
    "DatasetUpdateRow",
    "DatasetUpdateRowValues",
    "DatasetUpdateRowValuesAdditionalPropertyType3",
    "DatasetUpdatedAtSort",
    "DatasetUsedInProjectFilter",
    "DatasetVersionDB",
    "DatasetVersionIndexSort",
    "DeletePromptResponse",
    "DeleteRunResponse",
    "DeleteScorerResponse",
    "Document",
    "DocumentMetadata",
    "EditAction",
    "EditOverride",
    "EditOverrideEditSpansType0",
    "EditSpan",
    "EventStatus",
    "ExecutionStatus",
    "ExperimentCreateRequest",
    "ExperimentDataset",
    "ExperimentDatasetRequest",
    "ExperimentMetricsRequest",
    "ExperimentMetricsResponse",
    "ExperimentPhaseStatus",
    "ExperimentPlayground",
    "ExperimentPrompt",
    "ExperimentResponse",
    "ExperimentResponseAggregateFeedback",
    "ExperimentResponseAggregateMetrics",
    "ExperimentResponseRatingAggregates",
    "ExperimentResponseRatingAggregatesAdditionalProperty",
    "ExperimentResponseTags",
    "ExperimentStatus",
    "ExperimentUpdateRequest",
    "ExperimentsAvailableColumnsResponse",
    "ExtendedAgentSpanRecord",
    "ExtendedAgentSpanRecordDatasetMetadata",
    "ExtendedAgentSpanRecordFeedbackRatingInfo",
    "ExtendedAgentSpanRecordMetricInfoType0",
    "ExtendedAgentSpanRecordUserMetadata",
    "ExtendedAgentSpanRecordWithChildren",
    "ExtendedAgentSpanRecordWithChildrenDatasetMetadata",
    "ExtendedAgentSpanRecordWithChildrenFeedbackRatingInfo",
    "ExtendedAgentSpanRecordWithChildrenMetricInfoType0",
    "ExtendedAgentSpanRecordWithChildrenUserMetadata",
    "ExtendedLlmSpanRecord",
    "ExtendedLlmSpanRecordDatasetMetadata",
    "ExtendedLlmSpanRecordFeedbackRatingInfo",
    "ExtendedLlmSpanRecordMetricInfoType0",
    "ExtendedLlmSpanRecordToolsType0Item",
    "ExtendedLlmSpanRecordUserMetadata",
    "ExtendedRetrieverSpanRecord",
    "ExtendedRetrieverSpanRecordDatasetMetadata",
    "ExtendedRetrieverSpanRecordFeedbackRatingInfo",
    "ExtendedRetrieverSpanRecordMetricInfoType0",
    "ExtendedRetrieverSpanRecordUserMetadata",
    "ExtendedRetrieverSpanRecordWithChildren",
    "ExtendedRetrieverSpanRecordWithChildrenDatasetMetadata",
    "ExtendedRetrieverSpanRecordWithChildrenFeedbackRatingInfo",
    "ExtendedRetrieverSpanRecordWithChildrenMetricInfoType0",
    "ExtendedRetrieverSpanRecordWithChildrenUserMetadata",
    "ExtendedSessionRecord",
    "ExtendedSessionRecordDatasetMetadata",
    "ExtendedSessionRecordFeedbackRatingInfo",
    "ExtendedSessionRecordMetricInfoType0",
    "ExtendedSessionRecordUserMetadata",
    "ExtendedSessionRecordWithChildren",
    "ExtendedSessionRecordWithChildrenDatasetMetadata",
    "ExtendedSessionRecordWithChildrenFeedbackRatingInfo",
    "ExtendedSessionRecordWithChildrenMetricInfoType0",
    "ExtendedSessionRecordWithChildrenUserMetadata",
    "ExtendedToolSpanRecord",
    "ExtendedToolSpanRecordDatasetMetadata",
    "ExtendedToolSpanRecordFeedbackRatingInfo",
    "ExtendedToolSpanRecordMetricInfoType0",
    "ExtendedToolSpanRecordUserMetadata",
    "ExtendedToolSpanRecordWithChildren",
    "ExtendedToolSpanRecordWithChildrenDatasetMetadata",
    "ExtendedToolSpanRecordWithChildrenFeedbackRatingInfo",
    "ExtendedToolSpanRecordWithChildrenMetricInfoType0",
    "ExtendedToolSpanRecordWithChildrenUserMetadata",
    "ExtendedTraceRecord",
    "ExtendedTraceRecordDatasetMetadata",
    "ExtendedTraceRecordFeedbackRatingInfo",
    "ExtendedTraceRecordMetricInfoType0",
    "ExtendedTraceRecordUserMetadata",
    "ExtendedTraceRecordWithChildren",
    "ExtendedTraceRecordWithChildrenDatasetMetadata",
    "ExtendedTraceRecordWithChildrenFeedbackRatingInfo",
    "ExtendedTraceRecordWithChildrenMetricInfoType0",
    "ExtendedTraceRecordWithChildrenUserMetadata",
    "ExtendedWorkflowSpanRecord",
    "ExtendedWorkflowSpanRecordDatasetMetadata",
    "ExtendedWorkflowSpanRecordFeedbackRatingInfo",
    "ExtendedWorkflowSpanRecordMetricInfoType0",
    "ExtendedWorkflowSpanRecordUserMetadata",
    "ExtendedWorkflowSpanRecordWithChildren",
    "ExtendedWorkflowSpanRecordWithChildrenDatasetMetadata",
    "ExtendedWorkflowSpanRecordWithChildrenFeedbackRatingInfo",
    "ExtendedWorkflowSpanRecordWithChildrenMetricInfoType0",
    "ExtendedWorkflowSpanRecordWithChildrenUserMetadata",
    "FactualityTemplate",
    "FactualityTemplateResponseSchemaType0",
    "FeedbackAggregate",
    "FeedbackRatingDB",
    "FeedbackRatingInfo",
    "FeedbackType",
    "FewShotExample",
    "FileType",
    "FilterLeafLogRecordsFilter",
    "FilterParams",
    "FineTunedScorer",
    "FineTunedScorerAction",
    "FineTunedScorerResponse",
    "FineTunedScorerResponseClassNameToVocabIxType0",
    "FineTunedScorerResponseClassNameToVocabIxType1",
    "GeneratedScorerAction",
    "GeneratedScorerConfiguration",
    "GeneratedScorerResponse",
    "GeneratedScorerValidationResponse",
    "GenerationResponse",
    "GetIntegrationStatusIntegrationsNameStatusGetResponseGetIntegrationStatusIntegrationsNameStatusGet",
    "GetIntegrationsAndModelInfoForRunLlmIntegrationsProjectsProjectIdRunsRunIdGetGetRunIntegrationsResponse",
    "GetIntegrationsAndModelInfoLlmIntegrationsGetResponseGetIntegrationsAndModelInfoLlmIntegrationsGet",
    "GetProjectsPaginatedResponse",
    "GetProjectsPaginatedResponseV2",
    "GroundTruthAdherenceScorer",
    "GroundTruthAdherenceTemplate",
    "GroundTruthAdherenceTemplateResponseSchemaType0",
    "GroundednessTemplate",
    "GroundednessTemplateResponseSchemaType0",
    "GroupAction",
    "GroupCollaborator",
    "GroupCollaboratorCreate",
    "GroupMemberAction",
    "HTTPValidationError",
    "HallucinationSegment",
    "HealthcheckResponse",
    "Histogram",
    "HistogramBucket",
    "HistogramStrategy",
    "ImageGenerationEvent",
    "ImageGenerationEventImagesType0Item",
    "ImageGenerationEventMetadataType0",
    "InputMap",
    "InputPIIScorer",
    "InputSexistScorer",
    "InputSexistScorerType",
    "InputSexistTemplate",
    "InputSexistTemplateResponseSchemaType0",
    "InputToneScorer",
    "InputToxicityScorer",
    "InputToxicityScorerType",
    "InputToxicityTemplate",
    "InputToxicityTemplateResponseSchemaType0",
    "InputTypeEnum",
    "InsightType",
    "InstructionAdherenceScorer",
    "InstructionAdherenceTemplate",
    "InstructionAdherenceTemplateResponseSchemaType0",
    "IntegrationAction",
    "IntegrationDB",
    "IntegrationModelsResponse",
    "IntegrationName",
    "InternalToolCall",
    "InternalToolCallInputType0",
    "InternalToolCallMetadataType0",
    "InternalToolCallOutputType0",
    "InvalidResult",
    "InvokeResponse",
    "InvokeResponseHeadersType0",
    "InvokeResponseMetadataType0",
    "InvokeResponseMetricResults",
    "JobDB",
    "JobDBRequestData",
    "JobProgress",
    "LLMExportFormat",
    "LLMIntegration",
    "LabelStudioExportRequest",
    "LabelStudioIntegration",
    "LabelStudioIntegrationCreate",
    "LabelStudioIntegrationExtraType0",
    "LabelStudioProject",
    "LassoSelection",
    "LikeDislikeAggregate",
    "LikeDislikeRating",
    "ListDatasetParams",
    "ListDatasetProjectsResponse",
    "ListDatasetResponse",
    "ListDatasetVersionParams",
    "ListDatasetVersionResponse",
    "ListExperimentResponse",
    "ListGroupCollaboratorsResponse",
    "ListLogStreamResponse",
    "ListPromptDatasetResponse",
    "ListPromptTemplateParams",
    "ListPromptTemplateResponse",
    "ListPromptTemplateVersionParams",
    "ListPromptTemplateVersionResponse",
    "ListScorerVersionsResponse",
    "ListScorersRequest",
    "ListScorersResponse",
    "ListUserCollaboratorsResponse",
    "LlmMetrics",
    "LlmSpan",
    "LlmSpanDatasetMetadata",
    "LlmSpanToolsType0Item",
    "LlmSpanUserMetadata",
    "LogRecordsAvailableColumnsRequest",
    "LogRecordsAvailableColumnsResponse",
    "LogRecordsBooleanFilter",
    "LogRecordsBooleanFilterOperator",
    "LogRecordsCollectionFilter",
    "LogRecordsCollectionFilterOperator",
    "LogRecordsColumnInfo",
    "LogRecordsDateFilter",
    "LogRecordsDateFilterOperator",
    "LogRecordsDeleteRequest",
    "LogRecordsDeleteResponse",
    "LogRecordsExportRequest",
    "LogRecordsFilterType",
    "LogRecordsIDFilter",
    "LogRecordsIDFilterOperator",
    "LogRecordsMetricsQueryRequest",
    "LogRecordsMetricsResponse",
    "LogRecordsMetricsResponseAggregateMetrics",
    "LogRecordsMetricsResponseBucketedMetrics",
    "LogRecordsNumberFilter",
    "LogRecordsNumberFilterOperator",
    "LogRecordsQueryCountRequest",
    "LogRecordsQueryCountResponse",
    "LogRecordsQueryRequest",
    "LogRecordsQueryResponse",
    "LogRecordsSortClause",
    "LogRecordsTextFilter",
    "LogRecordsTextFilterOperator",
    "LogSpanUpdateRequest",
    "LogSpanUpdateResponse",
    "LogSpansIngestRequest",
    "LogSpansIngestResponse",
    "LogStreamCreateRequest",
    "LogStreamResponse",
    "LogStreamUpdateRequest",
    "LogTraceUpdateRequest",
    "LogTraceUpdateResponse",
    "LogTracesIngestRequest",
    "LogTracesIngestResponse",
    "LoggingMethod",
    "LunaInputTypeEnum",
    "LunaOutputTypeEnum",
    "MCPApprovalRequestEvent",
    "MCPApprovalRequestEventMetadataType0",
    "MCPApprovalRequestEventToolInvocationType0",
    "MCPCallEvent",
    "MCPCallEventArgumentsType0",
    "MCPCallEventMetadataType0",
    "MCPCallEventResultType0",
    "MCPListToolsEvent",
    "MCPListToolsEventMetadataType0",
    "MCPListToolsEventToolsType0Item",
    "ManualLlmValidateScorersLlmValidatePostBody",
    "Message",
    "MessageEvent",
    "MessageEventContentPartsType0Item",
    "MessageEventMetadataType0",
    "MessageRole",
    "MessagesListItem",
    "MessagesListItemRole",
    "MetaFilter",
    "MetadataFilter",
    "MetadataFilterOperator",
    "MetricComputation",
    "MetricComputationStatus",
    "MetricComputationValueType4",
    "MetricComputing",
    "MetricCritiqueColumnar",
    "MetricCritiqueContent",
    "MetricCritiqueJobConfiguration",
    "MetricError",
    "MetricFailed",
    "MetricNotApplicable",
    "MetricNotComputed",
    "MetricPending",
    "MetricSettingsRequest",
    "MetricSettingsResponse",
    "MetricSuccess",
    "MetricThreshold",
    "Metrics",
    "MetricsTestingAvailableColumnsRequest",
    "MistralIntegration",
    "MistralIntegrationCreate",
    "MistralIntegrationExtraType0",
    "Model",
    "ModelCostBy",
    "ModelType",
    "Name",
    "NodeNameFilter",
    "NodeNameFilterOperator",
    "NodeType",
    "NotNodeLogRecordsFilter",
    "NumericRollUpMethod",
    "NvidiaIntegration",
    "NvidiaIntegrationCreate",
    "NvidiaIntegrationExtraType0",
    "OpenAIFunction",
    "OpenAIIntegration",
    "OpenAIIntegrationCreate",
    "OpenAIIntegrationExtraType0",
    "OpenAIToolChoice",
    "OrNodeLogRecordsFilter",
    "OrganizationAction",
    "OutputMap",
    "OutputPIIScorer",
    "OutputSexistScorer",
    "OutputSexistScorerType",
    "OutputToneScorer",
    "OutputToxicityScorer",
    "OutputToxicityScorerType",
    "OutputTypeEnum",
    "OverrideAction",
    "PassthroughAction",
    "Payload",
    "Permission",
    "PreviewDatasetRequest",
    "ProjectAction",
    "ProjectBookmarkFilter",
    "ProjectBookmarkSort",
    "ProjectCollectionParams",
    "ProjectCreate",
    "ProjectCreateResponse",
    "ProjectCreatedAtFilter",
    "ProjectCreatedAtFilterOperator",
    "ProjectCreatedAtSortV1",
    "ProjectCreatorFilter",
    "ProjectCreatorFilterOperator",
    "ProjectDB",
    "ProjectDBThin",
    "ProjectDeleteResponse",
    "ProjectIDFilter",
    "ProjectIDFilterOperator",
    "ProjectItem",
    "ProjectLabels",
    "ProjectNameFilter",
    "ProjectNameFilterOperator",
    "ProjectNameSortV1",
    "ProjectRunsFilter",
    "ProjectRunsFilterOperator",
    "ProjectRunsSort",
    "ProjectType",
    "ProjectTypeFilter",
    "ProjectTypeFilterOperator",
    "ProjectTypeSort",
    "ProjectUpdate",
    "ProjectUpdateResponse",
    "ProjectUpdatedAtFilter",
    "ProjectUpdatedAtFilterOperator",
    "ProjectUpdatedAtSortV1",
    "PromptDatasetDB",
    "PromptInjectionScorer",
    "PromptInjectionScorerType",
    "PromptInjectionTemplate",
    "PromptInjectionTemplateResponseSchemaType0",
    "PromptOptimizationConfiguration",
    "PromptPerplexityScorer",
    "PromptRunSettings",
    "PromptRunSettingsResponseFormatType0",
    "PromptRunSettingsToolsType0Item",
    "PromptTemplateCreatedAtSort",
    "PromptTemplateCreatedByFilter",
    "PromptTemplateCreatedByFilterOperator",
    "PromptTemplateNameFilter",
    "PromptTemplateNameFilterOperator",
    "PromptTemplateNameSort",
    "PromptTemplateNotInProjectFilter",
    "PromptTemplateUpdatedAtSort",
    "PromptTemplateUsedInProjectFilter",
    "PromptTemplateVersionCreatedAtSort",
    "PromptTemplateVersionNumberSort",
    "PromptTemplateVersionUpdatedAtSort",
    "ProtectRequest",
    "ProtectRequestHeadersType0",
    "ProtectRequestMetadataType0",
    "ProtectResponse",
    "QueryDatasetParams",
    "ReasoningEvent",
    "ReasoningEventMetadataType0",
    "RecomputeLogRecordsMetricsRequest",
    "RecomputeSettingsLogStream",
    "RecomputeSettingsObserve",
    "RecomputeSettingsProject",
    "RecomputeSettingsRuns",
    "RegisteredScorer",
    "RegisteredScorerAction",
    "RegisteredScorerTaskResultResponse",
    "RenderTemplateRequest",
    "RenderTemplateResponse",
    "RenderedTemplate",
    "RetrieverSpan",
    "RetrieverSpanDatasetMetadata",
    "RetrieverSpanUserMetadata",
    "RollUpStrategy",
    "RollbackRequest",
    "RootType",
    "RougeScorer",
    "Rule",
    "RuleOperator",
    "RuleResult",
    "Ruleset",
    "RulesetResult",
    "RulesetsMixin",
    "RunCreatedAtFilter",
    "RunCreatedAtFilterOperator",
    "RunCreatedAtSort",
    "RunCreatedByFilter",
    "RunCreatedByFilterOperator",
    "RunDB",
    "RunDBThin",
    "RunIDFilter",
    "RunIDFilterOperator",
    "RunNameFilter",
    "RunNameFilterOperator",
    "RunNameSort",
    "RunParamsMap",
    "RunScorerSettingsPatchRequest",
    "RunScorerSettingsResponse",
    "RunTagCreateRequest",
    "RunTagDB",
    "RunUpdatedAtFilter",
    "RunUpdatedAtFilterOperator",
    "RunUpdatedAtSort",
    "ScoreAggregate",
    "ScoreRating",
    "ScorerConfig",
    "ScorerCreatedAtFilter",
    "ScorerCreatedAtFilterOperator",
    "ScorerCreatorFilter",
    "ScorerCreatorFilterOperator",
    "ScorerDefaults",
    "ScorerEnabledInRunSort",
    "ScorerLabelFilter",
    "ScorerLabelFilterOperator",
    "ScorerModelTypeFilter",
    "ScorerName",
    "ScorerNameFilter",
    "ScorerNameFilterOperator",
    "ScorerNameSort",
    "ScorerResponse",
    "ScorerTagsFilter",
    "ScorerTagsFilterOperator",
    "ScorerType",
    "ScorerTypeFilter",
    "ScorerTypeFilterOperator",
    "ScorerTypes",
    "ScorerUpdatedAtFilter",
    "ScorerUpdatedAtFilterOperator",
    "ScorersConfiguration",
    "Segment",
    "SegmentFilter",
    "SessionCreateRequest",
    "SessionCreateResponse",
    "SexistTemplate",
    "SexistTemplateResponseSchemaType0",
    "Split",
    "StageDB",
    "StageMetadata",
    "StageType",
    "StageWithRulesets",
    "StarAggregate",
    "StarAggregateCounts",
    "StarRating",
    "StepType",
    "StringData",
    "SubscriptionConfig",
    "SyntheticDataSourceDataset",
    "SyntheticDataTypes",
    "SyntheticDatasetExtensionRequest",
    "SyntheticDatasetExtensionResponse",
    "SystemMetricInfo",
    "TaggingSchema",
    "TagsAggregate",
    "TagsAggregateCounts",
    "TagsRating",
    "TaskResourceLimits",
    "TaskResultStatus",
    "TaskType",
    "TemplateStubRequest",
    "TestScore",
    "TextAggregate",
    "TextRating",
    "Token",
    "ToolCall",
    "ToolCallFunction",
    "ToolErrorRateScorer",
    "ToolErrorRateScorerType",
    "ToolErrorRateTemplate",
    "ToolErrorRateTemplateResponseSchemaType0",
    "ToolSelectionQualityScorer",
    "ToolSelectionQualityScorerType",
    "ToolSelectionQualityTemplate",
    "ToolSelectionQualityTemplateResponseSchemaType0",
    "ToolSpan",
    "ToolSpanDatasetMetadata",
    "ToolSpanUserMetadata",
    "ToxicityTemplate",
    "ToxicityTemplateResponseSchemaType0",
    "Trace",
    "TraceDatasetMetadata",
    "TraceMetadata",
    "TraceUserMetadata",
    "UncertaintyScorer",
    "UpdateDatasetContentRequest",
    "UpdateDatasetRequest",
    "UpdateDatasetVersionRequest",
    "UpdatePromptTemplateRequest",
    "UpdateScorerRequest",
    "UpsertDatasetContentRequest",
    "UserAction",
    "UserCollaborator",
    "UserCollaboratorCreate",
    "UserDB",
    "UserInfo",
    "UserRole",
    "ValidResult",
    "ValidateCodeScorerResponse",
    "ValidateLLMScorerLogRecordRequest",
    "ValidateLLMScorerLogRecordResponse",
    "ValidateRegisteredScorerResult",
    "ValidationError",
    "VegasGatewayIntegration",
    "VegasGatewayIntegrationCreate",
    "VegasGatewayIntegrationExtraType0",
    "VertexAIIntegration",
    "VertexAIIntegrationCreate",
    "VertexAIIntegrationExtraType0",
    "WorkflowSpan",
    "WorkflowSpanDatasetMetadata",
    "WorkflowSpanUserMetadata",
    "WriterIntegration",
    "WriterIntegrationCreate",
    "WriterIntegrationExtraType0",
)



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="BulkDeleteFailure")


@_attrs_define
class BulkDeleteFailure:
    """Details about a failed deletion.

    Attributes
    ----------
        dataset_id (str):
        dataset_name (str):
        reason (str):
    """

    dataset_id: str
    dataset_name: str
    reason: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        dataset_id = self.dataset_id

        dataset_name = self.dataset_name

        reason = self.reason

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"dataset_id": dataset_id, "dataset_name": dataset_name, "reason": reason})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        dataset_id = d.pop("dataset_id")

        dataset_name = d.pop("dataset_name")

        reason = d.pop("reason")

        bulk_delete_failure = cls(dataset_id=dataset_id, dataset_name=dataset_name, reason=reason)

        bulk_delete_failure.additional_properties = d
        return bulk_delete_failure

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
